;########################################################################
;#                                                                      #
;#                   ATTRIBUTES2.SCE                                    #
;#                                                                      #
;#  CONTENTS:                                                           #
;#      classes:  attributes-scenario                                   #
;#      parameters:                                                     #
;#               steps:  ask-for-specification, get-yes-answer,         #
;#                       get-specification, get-that-specification      #
;#                       get-none                                       #
;#     function:  make-attributes-scenario                              #
;#                                                                      #
;######################################################################## 

;;########################################################################
;##                                                                     ##
;##                     The Attributes Scenario                         ##     
;##                                                                     ##
;#########################################################################
;; The scenario is designed to be used to get the user to specify what item
;; they would like to order.  For example, if the user orders food, they are
;; asked to specify what kind of food.  Another term for this would be a 
;; clarification dialogue.


;**************************************
;*                                    *
;*      ATTRIBUTES-SCENARIO CLASS     *
;*                                    *
;* object:  the orderable item that   *
;*          cause the need for the    *
;*          scenario to be created    *
;**************************************

(defclass attributes-scenario (scenario)
((object :initarg :my-object
         :accessor my-object
         :initform nil)
  ))





(defparameter ask-for-specification (make-instance 'mystep
;; Asks user to specific what kind of an orderable item they would like

                      :name "ask-for-specification"
                      :my-condition
                       (notc (lambdac #'(lambda (rest mystep)
                                      (prev-step-matches rest '
                                         ("ask-for-specification" "kinds-of" 
 						"describe-item") 2))))
            :my-action #'(lambda (self restaurant)
             (let* ((current-scene (current-scenario (waiter restaurant)))
                    (current-object (my-object current-scene)))
;;          (format t "~%Current-object is: ~a" current-object)
;;          (format t "~%current-scene is: ~a" current-scene)
                (cond ((eq (length (subclasses current-object)) 1)
                    (say (waiter restaurant) "~% Is ~a O.K.?" 
                          (car (subclasses current-object))))
                (t (say (waiter restaurant) "~%What kind of ~a would you like?"
                              (reference-name current-object) )
                    (say (waiter restaurant) "~%We only have")
                             
                         (loop for  i from 0 to 
                              (- (length (subclasses current-object)) 2) do
                                (say (waiter restaurant) " ~a," 
                                   (nth i (subclasses current-object)))
                          )
                             (say (waiter restaurant) " and ~a. " 
                                (nth (- (length (subclasses current-object)) 1)
                                         (subclasses current-object)))))
                   (setf (expecting-input (waiter restaurant)) t)
))))

(defparameter get-yes-answer (make-instance 'mystep
;; Right now this step doesn't actually work, but since all orderable item
;; have either no subclasses or at least two subclasses, the step is never used
                    :name "get-yes-answer"
                    :input-condition (wordc :nec '(yes))
                    :my-condition (lambdac #'(lambda (rest mystep)
                       (prev-step-matches rest '("ask-for-specification") 2)))
                    :my-action #' (lambda (self restaurant)
                      (add-to-order-pad (make-instance '(make-symbol 
                            (car (subclasses (my-object (current-scenario 
                                  (waiter restaurant))))) restaurant))))))

(defparameter get-specification (make-instance 'mystep
;; This step is almost exactly the same as the get-item step
                    :name "get-specification"
                    :input-condition (orc (list
                          (phrasec
                              (list
                                 anystring :nec 'customer
                                                (wildcard 0 2)
                                       '(want have order)
                                         :nec focus-np-cond))
                               (phrasec (list
                                   anystring :nec focus-np-cond))
                              ))
                          
                    :my-condition (orc (list
                               (spscorec :sc 2 :nec
                             (wordc :nec
                                   '(orderable-item)))
                          (lambdac #'(lambda (rest mystep)
                       (prev-step-matches rest '("ask-for-specification" 
                         "kinds-of" "describe-item" "ask-rephrased") 2)))
                           ))

                    :my-action #' (lambda (self restaurant)
              ;;  (format t "~% new input is: ~a" (input (waiter restaurant)))
                       (let ((item-to-add (binding-values self 'focus))
                             (input-item '()))
                          (loop for item in item-to-add do
                            (setf input-item (initialize 
                                (make-instance (obj-class item))))
                     
       (setf (reference-name input-item) (string-downcase (form-in-text item)))
                      
                      (add-to-order-pad (order-pad restaurant) input-item)
                      (remove-item (order-pad restaurant) 
                           (my-object (current-scenario (waiter restaurant))))
                         (cond ((null (subclasses input-item))
                         
                         (format t "~%~a? Sure!" (reference-name input-item))
                          (cond ((not (eq (subscenario input-item) nil))
                              (setf (item (subscenario input-item)) input-item)
                                    (push (subscenario input-item) 
                                     (scenario-stack (waiter restaurant))))))
                            
                         
                         (t (make-attributes-scenario input-item restaurant)
;;                       (format t "~%MAKING-ATTRIBUTES-SCENARIO")
))
       
                         
                       )))))

(defparameter get-that-specification (make-instance 'mystep
;;This is for if the user's say something line I'll have that.
                      
       :name "get-that-specification"
      :input-condition  (wordc :nec '(that)
                               :sup '(customer have))
      :my-condition  (lambdac #'(lambda (rest mystep)
                       (let ((results (prev-step-matches rest 
                                        '( "describe-item" "tell-cost" 
                                                          "kinds-of") 4))
                                (prev-step (last-step (waiter rest))))
                                (when (success results)
                                (setf (bindings results)(bindings 
                                (input-results prev-step)))
                            (setf (bindings results) 
                            (append (bindings (results prev-step))
                                   (bindings results))))
                       results)))
					
      :my-action #'(lambda (self restaurant)
                      (let ((item-to-add (initialize 
                                           (make-instance 
                                               (obj-class 
                                               (binding-value self 'focus))))))
                         (setf (reference-name item-to-add)
                               (string-downcase 
                                (form-in-text (binding-value self 'focus))))
                         (add-to-order-pad (order-pad restaurant) item-to-add)
                         (remove-item (order-pad restaurant) 
                           (my-object (current-scenario (waiter restaurant))))
                         
                         (cond ((null (subclasses item-to-add))
                         (format t "~%~a? Sure!" (reference-name item-to-add))
                           (cond ((not (eq (subscenario item-to-add) nil))
                          (setf (item (subscenario item-to-add)) item-to-add)
                           (push (subscenario item-to-add) 
                                (scenario-stack (waiter restaurant))))))
                        (t (make-attributes-scenario item-to-add restaurant)))
                    

))))


(defparameter get-none (make-instance 'mystep
;; This step is basically a default step that is invoking if the user does
;; not chose an understandable option
            :name "get-none"
            :input-condition (andc :nec (list
                                     (notc (wordc :nec '(that)))
                                    (notc (wordc :nec '(orderable-item)))
                                    (notc (wordc :nec '(non-orderable-item)))
                                    (notc (wordc :nec '(yes)))))
            :my-condition (spscorec :sc 1)
            :my-action #'(lambda (self restaurant)
                (cond ((kind-of (class-name (class-of (my-object 
                         (current-scenario (waiter restaurant))))) 'drink-item)
                (add-to-section (order-pad restaurant) "drink-items"  
                                             (make-instance 'nothing)))
               
		  ((kind-of (class-name (class-of (my-object (current-scenario (waiter restaurant))))) 'food-item)
                (add-to-section (order-pad restaurant) "main-course" 
                                                 (make-instance 'nothing)))
                 
                  ((kind-of (class-name (class-of 
                       (my-object (current-scenario (waiter restaurant))))) 
                                             'dessert-item)
                (add-to-section (order-pad restaurant) "dessert-items" 
                                                 (make-instance 'nothing))))

                (remove-item (order-pad restaurant) (my-object 
                                (current-scenario (waiter restaurant))))
                (decrement-counters (order-pad restaurant) 
                      (my-object (current-scenario (waiter restaurant))))

                (say (waiter restaurant) 
                        "~%Hmm..I guess none of the above then.")
)))



(defun make-attributes-scenario (object restaurant)
;; This function creates an attributes scenario and pushes it onto the stack.
(let ((new-scenario (make-instance 'attributes-scenario
                               :my-object object
                               
                               :name "attributes-scenario"
                               :steps (list invalid-order ask-rephrased 
                                             kinds-of describe-item tell-cost 
                                            describe-item ask-for-specification
                                            get-none get-specification 
                                           get-yes-answer 
                                               get-that-specification)
                                :end-condition
                           (lambdac #'(lambda (restaurant mystep)
        
 ;;                           (format t "~%Trying to test end-condition")
                            (prev-step-matches restaurant '("get-specification"
"get-that-specification" "get-yes-answer" "get-none") 2)))
                            )))
                                
(push new-scenario (scenario-stack (waiter restaurant)))
new-scenario
))
















