;######################################################################
;#                                                                    #
;#                     CONTEXT.LSP                                    #
;#                                                                    #
;#   CONTENTS:                                                        #
;#      class;  context-error                                         #
;#      Comments:  This file is used for the post-dialogue analysis.  #
;######################################################################

;************************************************************************
;*                                                                      *
;*               CONTEXT-ERROR CLASS                                    *
;*                                                                      *
;*  This class will be used to keep track of errors the user makes      *
;* with regards to the correct actions to perform in certain situtation *
;* For example the user shouldn't order food before the are seated and  *
;* so on.                                                               *      
;************************************************************************

(defclass context-error ()
  ((starting-context
    :initarg :starting-context
    :accessor starting-context
    :initform '())
   (next-context
    :initarg :next-context
    :accessor next-context
    :initform '())
   (last-step
    :initarg :last-step
    :accessor last-step
    :initform '())
   (next-step
    :initarg :next-step
    :accessor next-step
    :initform '())
   (jump
    :initarg :jump
    :accessor jump
    :initform "")
   (problem
    :initarg :problem
    :accessor problem
    :initform "")
   (input
    :initarg :input
    :accessor input
    :initform "")
   (line-num
    :initarg :line-num
    :accessor line-num
    :initform 0)
   ))


;*************************************
;*                                   *
;* methods for context-errors class  *
;*                                   *
;*************************************

(defmethod determine-jump ((self context-error))
;;  This method determines from the instance variable the type of situation
;; error that has been made.
  (let ((start (name (starting-context self)))
      (end (name (next-context self))))
    (cond ((equal start "simple greeting scenario")
	   (cond ((equal end "drink-order")
		  (setf (jump self) "greeting-drink"))
		 ((equal end "food-order")
		  (setf (jump self) "greeting-food"))
		 ((equal end "dessert-order")
		  (setf (jump self) "greeting-dessert"))))
	  ((equal start "drink-order")
	   (cond ((equal end "food-order")
		  (setf (jump self) "drink-food"))
		 ((equal end "dessert-order")
		  (setf (jump self) "drink-dessert"))
		 ))
	  ((equal start "food-order")
	   (cond ((equal end "drink-order")
		  (setf (jump self) "food-drink"))))
	  ((equal start "dessert-order")
	   (cond ((equal end "drink-order")
		  (setf (jump self) "dessert-drink"))
		 ((equal end "food-order")
		  (setf (jump self) "dessert-food")))))))

(defmethod determine-problem ((self context-error))
;;  This method determines which method to call based on what error has been
;; made.
  (cond ((equal (jump self) "greeting-drink")
	 (greeting-drink self))
	((equal (jump self) "greeting-food")
	 (greeting-food self))
	((equal (jump self) "greeting-dessert")
	 (greeting-dessert self))
	((equal (jump self) "drink-food")
	 (drink-food self))
	((equal (jump self) "drink-dessert")
	 (drink-dessert self))
	((equal (jump self) "dessert-drink")
	 (dessert-drink self))
	((equal (jump self) "food-drink")
	 (food-drink self))
	((equal (jump self) "dessert-food")
	 (dessert-food self))))


;;  The rest of the methods for this class just figure out what type of
;; error message to write.


(defmethod greeting-drink ((self context-error))

  (cond ((equal (next-step self) "get-item")
	 (setf (problem self) 
"While in the greeting stage of the restaurant visit there was a jump into the 
drink  ordering scenario. 
It is best not to try to order a drink until you are seated."))
        ((equal (last-step self) "ask-drink-order")
         (setf (problem self) ""))
	(t (setf (problem self) 
"While in the greeting stage of the restaurant visit there was a jump into the 
drink ordering scenario.
")))
  )

(defmethod greeting-food ((self context-error))
 
  (cond ((equal (next-step self) "get-item")
	 (setf (problem self)  
"While in the greeting stage of the restaurant visit there was a jump into the 
food ordering scenario. 
It is best not to try to order something to eat until you have been seated.  
Ideally, you would wait until you have been asked."))
	((equal (last-step self) "ask-food-order")
         (setf (problem self) ""))
	(t  (setf (problem self) 
"While in the greeting scenario there was a jump into the food ordering scenario.")))

)

(defmethod greeting-dessert ((self context-error))
 
  (cond ((equal (next-step self) "get-item")
	 (setf (problem self) 
"While in the greeting stage of the restaurant visit there was a jump into the 
dessert ordering scenario. 
It is best not to try to order dessert until you have been seated.  
Ideally you would waited until you have finished your meal and have been 
asked if you would like to order dessert."))
        ((equal (last-step self) "ask-dessert-order")
         (setf (problem self) ""))
        (t (setf (problem self) 
"While in the greeting scenario there was a jump into the dessert ordering scenario.")
	   )))

(defmethod drink-food ((self context-error))

  (cond ((and (equal (last-step self) "ask-drink-order") 
	      (equal (next-step self) "get-item"))
	 (setf (problem self) 
"While in the drink ordering scenario there was a jump into the food ordering 
scenario.
In general is would be best to wait until you are asked what you would like
to eat before you order food."))
	((equal (last-step self) "ask-food-order")
         (setf (problem self) ""))
	(t  (setf (problem self) 
"While in the drink ordering scenario there was a jump into the food ordering 
scenario."))))


(defmethod drink-dessert ((self context-error))

  (cond ((and (equal (last-step self) "ask-drink-order") 
	      (equal (next-step self) "get-item"))
	 (setf (problem self) 
"While in the drink ordering scenario there was a jump into the dessert ordering 
scenario.
It is best to wait until you are asked if you would something for dessert
before trying to order it."))
	((equal (last-step self) "ask-dessert-order")
         (setf (problem self) ""))
	(t (setf (problem self) 
"While in the drink ordering scenario there was a jump into the dessert ordering 
scenario."))))


(defmethod dessert-drink ((self context-error))
  (setf (problem self) 
"While in the dessert ordering scenario there was a jump into the drink 
ordering scenario."))

(defmethod dessert-food ((self context-error))
  (setf (problem self) 
"While in the dessert ordering scenario there was a jump into the food 
ordering scenario."))



(defmethod food-drink ((self context-error))
  (setf (problem self) 
"While in the food ordering scenario there was a jump into the drink ordering 
scenario."))








