;######################################################################
;#                                                                    #
;#                DICTIONA.LSP                                        #
;#                                                                    #
;# CONTENTS:                                                          #
;#     Classes:  dictionary, word, noun, verb, determiner, punctuation#
;#     Comment:  The methods for the dictionary class are what do the #
;#               parsing.                                             #
;#                                                                    #
;######################################################################

                                                    
;********************************************
;*        DICTIONARY CLASS                  *
;*                                          *
;*  This is just where the words are stored *
;********************************************
(defclass dictionary ()
    ((contents
        :initarg :contents
        :accessor contents
        :initform '())))

;*******************************************
;*                                         *
;*          WORD CLASS                     *
;*                                         *
;* base-form: the word itself              *
;* special-form:  equivalent words         *
;* relevant-scenario:  possible scenarios  *
;*                     to go to            *
;* description:  most for orderable items  *
;* form-in-text:  the form the user gave   *
;*******************************************

(defclass word ()
    (
     (base-form
      :initarg :base-form
      :accessor base-form
      :initform nil)
     (special-forms
      :initarg :special-forms
      :accessor special-forms
      :initform '())
     (pos
      :initarg :pos
      :accessor pos
      :initform "other")
     (relevant-scenarios
      :initarg :relevant-scenarios
      :accessor relevant-scenarios
      :initform '())
     (special-phrases
      :initarg :special-phrases
      :accessor special-phrases
      :initform '())
     (description
      :initarg :description
      :accessor description
      :initform "")
     (form-in-text
      :initarg :form-in-text
      :accessor form-in-text
      :initform "")))

;***************************
;*                         *
;*    NOUN CLASS           *
;*                         *
;* for orderable items     *
;***************************

(defclass noun (word)
      ((object-class
        :initarg :obj-class
        :accessor obj-class
        :initform 'nil)
       (pos   
        :initform "noun")))

;**************************
;*                        *
;*     PRONOUN CLASS      *
;*                        *
;**************************
   
(defclass pronoun (noun)
  ((pos :initform "pronoun")))


;**************************
;*                        *
;*     VERB CLASS         *
;*                        *
;**************************

(defclass verb (word)
    ((theta-struct
      :initarg :theta-struct
      :accessor theta-struct
      :initform '())
     (pos
      :initform "verb")))

;**************************
;*                        *
;*     DETERMINER CLASS   *
;*                        *
;**************************

(defclass determiner (word)
    ((pos :initform "determiner")
     (definite 
        :initarg :definite
        :accessor definite
        :initform nil)
     (number
        :accessor number
        :initarg :number
        :initform 1)))

;**************************
;*                        *
;*     PUNCTUATION CLASS  *
;*                        *
;**************************

(defclass punctuation (word) ())

(defmethod look-up ((self dictionary) index input-in-word-segments)
   ;;;The input is a stream of lists of characters, and the index tells
   ;;;us which word to look at next. If the word at the index (or some
   ;;;substream of the input) matches something in the dictionary, then
   ;;;return a list containing first the new index (the old one plus the
   ;;;number of words recognized) and second the word that was recognized.
   (let ((curword '())
         (found '())
         (substring (nth-cdr index input-in-word-segments)))
      (loop for current-word in (contents self) until found do
         (setf found (word-matches current-word substring))
            (if found (setf (car found) (+ index (car found)))))
    ;; (format t "~% Found is: ~a" found)
      found))

(defmethod initialize ((self dictionary))
    ;;;The idea of this method is to take each words' supplementary
    ;;;forms and change them from strings to lists. This would save one
    ;;;step in the word-matches procedure and might make the parsing 
    ;;;faster and less redundant.
   (loop for w in (contents self) do
    ;; (format t "~%Looping with word: ~a" w)
     (loop for i from 0 below (length (special-forms w)) do
      ;;(format t "~% looping with special-form: ~a" (nth i (special-forms w)))
        (setf (nth i (special-forms w)) 
              (walk-string recognize-words (nth i (special-forms w))))))
   self
)

(defmethod size ((self dictionary))
   (format t "~%Number of entries: ~a" (length (contents self)))
   (let ((word-count 0))
      (loop for w in (contents self) do
         (incf word-count)
         (setf word-count (+ word-count (length (special-forms w)))))
      (format t "~%Number of words and phrases recognized: ~a" word-count)))

(defmethod word-copy ((self word) &key actual-input)
   (let ((a
            (make-instance (class-of self)
               :form-in-text actual-input
               :base-form (base-form self)
               :special-forms (special-forms self)
               :pos (pos self)
               :relevant-scenarios (relevant-scenarios self)
               :special-phrases (special-phrases self)
               :description (description self))))
      (cond  ((or (typep self 'noun) (typep self 'pronoun))
              (setf (obj-class a) (obj-class self)))
            ((typep self 'determiner)
             (setf (definite a) (definite self)))
            (t))
      a)
   )
      
;;;(defmethod word-matches ((self word) list)
;;;   (let ((word (string-downcase (list-to-string list))))
;;;      (or (string= word (string-downcase (string (base-form self))))
;;;          (member word (special-forms self) :test #'string=))))

(defmethod word-matches ((self word) substring-input)
    (let ((results '())
          (found '()) )
     ;;  (format t "~%inside word-matches with ~a and ~a" self substring-input)
       (setf found (lines-up (walk-string recognize-words 
                               (string-downcase (string (base-form self))))
                             substring-input))
       (cond (found (setf results 
                     (list found 
                        (word-copy self :actual-input 
                                   (string-downcase 
                                      (string (base-form self))) ) )))
             (t (loop for w in (special-forms self) until found do
                 (setf found (lines-up w substring-input))
                 (if found (setf results 
                             (list found 
                                   (word-copy self
                                     :actual-input (list-to-string w))))))))
       results)) 
       


(defmethod get-class-name ((self word))
  (if (or (typep self 'noun) (typep self 'pronoun))
      (obj-class self)
  (base-form self)))

(defmethod print-object ((self word) stream)
  (format stream "<~a: ~a>" (class-name (class-of self)) (base-form self))) 

