;-----------------------------------------------------------------------
; Non-orderable-item class
; 
; This class is used to represent things that could be ordered in a restaurant
; but not in this particular restaurant.
; 
;-----------------------------------------------------------------------
(defclass non-orderable-item ()
  ((reference-name :accessor reference-name
                   :initform ""
                 :initarg :reference-name)))
 

(defclass alcoholic-beverage (non-orderable-item)
())

(defclass other-drinks (non-orderable-item)
())

(defclass chicken (non-orderable-item)
())

(defclass other-pastas (non-orderable-item)
())

(defclass other-meats (non-orderable-item)
())

(defclass pizza (non-orderable-item)
())

(defclass sandwich (non-orderable-item)
())

(defclass ginger-ale (non-orderable-item)
())

(defclass salad (non-orderable-item)
())

(defclass cream-soda (non-orderable-item)
())

(defclass pie (non-orderable-item)
())

(defclass donut (non-orderable-item)
())






;-----------------------------------------------------------------------
; Generic orderable-item class
; 
; The description gets printed out when the student asks what an item is.
; Subscenario is the name if the scenario that will be put on the stack
; if the item in ordered.  The subscenario will contains steps neccessary
; to find out an record additional information about an item.  For example
; would you like cream and sugar with your coffee?
; Subclasses are used to answer the question like "what kinds of X do
; you have?"
; The reference-name is a string that the waiter can use to remember what
; the student called something. For example, the student may have ordered
; "pepsi" which is a COLA object. Instead of calling it "COLA", it would
; be nice to call it "pepsi". 
;------------------------------------------------------------------------
(defclass orderable-item ()
  ((cost :accessor cost 
         :initform 0.00 
         :initarg :cost)
   (description :accessor description 
                :initform "An orderable item"
                :initarg :description)
    (subclasses :accessor subclasses
               :initform '("food" "drinks" "dessert")
               :initarg :subclasses)
   (reference-name :accessor reference-name
                   :initform ""
                   :initarg :reference-name)
      (subscenario :accessor subscenario
                 :initform nil
                 :initarg :subscenario)
   ))

;----------------
; Object methods
;----------------
(defmethod initialize ((self orderable-item))
;  (setf (parts self) (make-hash-table :size 7))
  self)

(defmethod has-slots ((self orderable-item))
  (> (hash-table-count (parts self)) 0))
         


(defmethod print-object ((self orderable-item) stream)
  (format stream "~a" (type-of self)))

(defmethod dump-object ((self orderable-item) stream)
   (format stream "~%      ")
   (print-object self stream)
   (maphash 
       #'(lambda (key value)
          (format stream "~%         ~a ~a" key value))
       (parts self)))

;(defmethod set-slots ((self orderable-item))
;   (maphash 'fill-slot (parts self)))

;(defun fill-slot (slot-name value-range)
;    (cond ((equal value-range yes-no)
;           (format t "Would you like ~a? ~%" slot-name)
;           (format t "THIS STATEMENT SHOULD BE REPLACED BY CODE THAT GETS THE YES OR NO"))
;          (t (format t "Would you like ")
;             (loop for n in value-range do
;                   (format t "~a, " n))
;             (format t "or ~a?" (first (last value-range)))
;             (format t "SUBSTITUTE THIS STATEMENT WITH A CALL TO A STEP THAT PICKS OFF 
;                THE PERSON'S CHOICE OF ITEMS"))))

;------------------------------------
; Some useful slot-filler definitions
;------------------------------------
(defparameter yes-no (list 'yes 'no))

(defun items-match (input-word symbol)
   ;;The input word can be either a word or a list of characters. The
   ;;symbol must be a symbol.
   (if (listp input-word)
      (string-equal (list-to-string input-word) (string symbol))
      (string-equal (form-in-text input-word) (string symbol))))
   
(defmethod slot-filled ((self orderable-item) slot)
   :description "Checks whether the slot position has been filled. If the slot has a 
      list as its value, the answer is no. Any other object (other than a list) is 
      assumed to be a slot-filler. NOTE: NULLS COUNT AS SLOT-FILLERS."
   (or (null (gethash slot (parts self)))
       (not (listp (gethash slot (parts self))))))
             
(defun find-slot-fillers (object input restaurant)
   :description "Checks the input stream for any slot-fillers that the
    student might have entered. If there are any present, this method 
    fills them in. If there is no slot-filler for a slot, then this method
    creates a slot-filler scenario to ask about that (those) slot-filler(s). "
   (let ((no-found (found-word 'no input 0)) 
         (no-before-input '())
         (remaining-result '#(lambda (result)
                                ;;when found-word is used, the result is a pair where the first
                                ;;item is the list of input found after the word. 
                                (car result)))
      ;   (new-slot-scenario (make-slot-scenario object restaurant))
      ;   (need-slot-scenario nil)
         )  
      
   (maphash 
      #'(lambda (key value)
      
   
           (cond ((equal value yes-no) 
                  ;;;This is for yes-no types of slot-fillers. If you have this kind, then you look 
                  ;;;for the key-name (eg. cream) instead of a value, which is yes or no (since yes or no
                  ;;;are considered meaningless out of context). If you find a "no" preceeding the key then 
                  ;;;assume the structure is "no sugar" or something like that. (OBVIOUS LINGUISTIC 
                  ;;;PITFALLS TO THIS APPROACH - eg. "I don't want cream but I'd sure appreciate 
                  ;;; sugar in my coffee!")
                 (loop for input-word in input do
                     (when (items-match input-word key)
                            ;;Two possibilities: there is a "no" in front of it or there is not. If there 
                            ;;is no "no", it cannot be in front. If the "no" is not in front of the
                            ;;input word, then put a "yes" into the slot-value at the current key.
                            (when no-found 
                                   (loop for i-word in (car no-found) do
                                      (when (items-match i-word key)
                                         (setf (gethash key (parts object)) no)
                                        ; (format t "~%found a word after the no: ~a" key)
                                      )))
                           
                            (when (equal (gethash key (parts object)) yes-no) ;;i.e. it hasn't been changed yet
                                 (setf (gethash key (parts object)) yes))))
                  ;(when (equal (gethash key (parts object)) yes-no) ;;;i.e. we didn't find the keyword in the input
                  ;   (setf need-slot-scenario t))
                   )
                                    
                 (t
                    ;;;This is for multivalued slot-fillers (eg. flavour of pop). Look for any of 
                    ;;; the possible slot-fillers in the input. If you find one, use it. ASSUMPTIONS: 
                    ;;;NOBODY WILL ASK FOR TWO DIFFERENT SOFT DRINKS AT THE SAME TIME. THIS WOULD 
                    ;;;REQUIRE CHECKING THE INPUT REMAINING AFTER ONE PARSE. WHILE NOT 
                    ;;; IMPOSSIBLE, THIS REQUIRES MORE THOUGHT THAN I'M UP TO RIGHT NOW.  
                    (let ((found nil)) 
                    (loop for word in value until found do
                       (loop for input-word in input until found do
                         (cond ((items-match input-word word)
                                  (setf (gethash key (parts object)) word)
                                 (setf found t)) 
;;;                                 ((and (equal (class-of input-word)
;;;                                          (find-class 'word))
;;;                                       (equal word (form-in-text input-word)))
;;;                                  ;;;The input word was parsed. 
;;;                                  (format t "~%Found a slot-filler value: ~a"
;;;                                     word)
;;;                                  (setf (gethash key (parts object)) word))
                                ) ))
                       ;(when (not found)
                       ;   (setf need-slot-scenario t))
                       ))
                    ))
      ;)
      (parts object))
      ;;(when need-slot-scenario
      ;;   (push new-slot-scenario (scenario-stack (waiter restaurant))))
      ))
;-------------------------------
; the "nothing" object
;-------------------------------
(defclass nothing (orderable-item)
( (reference-name :initform "nothing")))                                
;-------------------------------
; drink-items
;------------------------------
(defclass drink-item (orderable-item)
  ((subclasses :initform '("hot drinks" "cold drinks"))
   ))

(defclass hot-drink (drink-item)
  ((subclasses :initform '("coffee" "tea" "hot-chocolate"))
   ))

(defclass coffee (hot-drink)
  ((cost :initform 1.25)
   (subclasses :initform '())
   (subscenario :initform coffee-scenario)
  (description 
   :initform "Coffee is a hot, bitter drink usually served with milk and sugar.")))

;;(defmethod initialize :after ((self coffee))
  ;;(setf (gethash 'cream (parts self)) cream)
 ;; (setf (gethash 'sugar (parts self)) sugar)
 ;; self)

;;(defparameter cream  yes-no)
;;(defparameter sugar yes-no)
 
(defclass tea (hot-drink)
   ((cost :initform 1.25)  
    (subclasses :initform '())
   (description :initform 
       "Tea is a hot drink made from plant leaves. Black tea is from China.")))

(defclass hot-chocolate (hot-drink)
  ((cost :initform 1.25)
   (subclasses :initform '())
   (on-menu :initform nil)
   (description :initform
       "Hot chocolate is a milky, chocolate flavoured hot drink.")))
 
(defclass cold-drink (drink-item) 
  ((subclasses :initform '("soft-drink" "milk" "iced-tea" "water"))))


(defclass water (cold-drink)
((cost :initform 0.00)
 (subclasses :initform '())))

(defclass milk (cold-drink)
((cost :initform 1.00)
 (subclasses :initform '())))

(defclass iced-tea (cold-drink)
((cost :initform 1.25)
 (subclasses :initform '())))

(defclass soft-drink (cold-drink)
    ((cost :initform 1.25)
     (subscenario :initform pop-scenario)
     (subclasses :initform '("cola" "sprite" "orange-crush" "root-beer"))
      (description :initform "Pop is carbonated water with flavouring. We have cola, lemon-lime, and 
orange flavours.")))

;;(defmethod initialize :after ((self soft-drink))
;;   (setf (gethash 'diet (parts self)) '("diet" "regular")))


(defclass cola (soft-drink)((subclasses :initform '())
))
(defclass sprite (soft-drink)((subclasses :initform '())
))
(defclass orange-crush (soft-drink)((subclasses :initform '())
))
(defclass root-beer (soft-drink)((subclasses :initform '())
))
      
;-------------------------------
; food
;-------------------------------
(defclass food-item (orderable-item)
 ((subclasses :initform '("fries" "burgers" "pasta" "steak"))))

(defclass pasta (food-item)
((subclasses :initform '("spaghetti" "ravioli"))
(cost :initform 0.0)))

(defclass steak (food-item)
((subclasses :initform '())
 (cost :initform 8.00)
 (subscenario :initform steak-scenario)))

(defclass ravioli (pasta)
((subclasses :initform '())
 (cost :initform 5.00)))
 
 (defclass spaghetti (pasta)
  ((subclasses :initform '())
   (cost :initform 5.00)))

(defclass fries (food-item)((subclasses :initform '())
                             (cost :initform 3.00)))
(defclass burger (food-item) ((subclasses :initform '("cheeseburger" "deluxe-burger" "hamburger"))))
                               
(defclass cheeseburger (burger) ((subclasses :initform '())
                                    (cost :initform 4.00)))

(defclass deluxe-burger (burger) ((subclasses :initform '())
                                 (cost :initform 4.50)))

(defclass hamburger (burger) ((subclasses :initform '())
                                  (cost :initform 3.50)))

(defclass dessert-item (orderable-item)
  ((subclasses :initform '("cake" "ice-cream"))))

(defclass cake (dessert-item)((subclasses :initform '("chocolate cake" "cheesecake"))
                             (cost :initform 1.50)))

(defclass chocolate-cake (cake)
((subclasses :initform '())
))

(defclass cheesecake (cake)
((subclasses :initform '())))


(defclass ice-cream (dessert-item) ((subclasses :initform '("vanilla ice-cream" "strawberry ice-cream"))
                                             (cost :initform 1.50)))

(defclass vanilla-ice-cream (ice-cream)
((subclasses :initform '())
))

(defclass strawberry-ice-cream (ice-cream)
((subclasses :initform '()))) 

;-------------------------------
; people
;-------------------------------

(defclass person ()((subclasses :initform '(customer))))
(defclass customer (person)((subclasses :initform '())))


























