;#########################################################################
;#                                                                       #
;#                      OBJECTS2.LSP                                     #
;#                                                                       #
;#    CONTENTS:                                                          #
;#         Classes:  non-orderable-item                                  #
;#                            :  alcoholic-beverage, other-drinks,       #
;#                               chicken, other-pastas, other-meats,     #
;#                               pizza, sandwich, ginger-ale, cream-soda #
;#                               pie, donut                              #
;#                  orderable-item                                       #
;#                           : nothing                                   #
;#                           :drink-item                                 #
;#                                 : hot-drink                           #
;#                                      :coffee, tea, hot-chocolate      #
;#                                 : cold--drink                         #
;#                                      :water, milk, iced-tea,          #
;#                                       soft-drink                      #
;#                                          :cola, sprite, orange-crush, #
;#                                           root-beer                   #
;#                           :food-item                                  #
;#                                :steak, fries, pasta                   #
;#                                                :ravioli, spaghetti    #
;#                                 burger                                #
;#                                     :cheeseburger, deluxe-burger,     #
;#                                      hamburger                        #
;#                           :dessert-item                               #
;#                                :cake                                  #
;#                                    :chocolate-cake, cheesecake        #
;#                                :ice-cream                             #
;#                                    :vanilla-ice-cream,                #
;#                                     strawberry-ice-cream              #
;#                  people                                               #
;#                      :customer                                        #
;#                                                                       #
;#########################################################################

;-----------------------------------------------------------------------
; Non-orderable-item class
; 
; This class is used to represent things that could be ordered in a restaurant
; but not in this particular restaurant.
; 
;-----------------------------------------------------------------------
(defclass non-orderable-item ()
  ((reference-name :accessor reference-name
                   :initform ""
                 :initarg :reference-name)))
 

(defclass alcoholic-beverage (non-orderable-item)
())

(defclass other-drinks (non-orderable-item)
())

(defclass chicken (non-orderable-item)
())

(defclass other-pastas (non-orderable-item)
())

(defclass other-meats (non-orderable-item)
())

(defclass pizza (non-orderable-item)
())

(defclass sandwich (non-orderable-item)
())

(defclass ginger-ale (non-orderable-item)
())

(defclass salad (non-orderable-item)
())

(defclass cream-soda (non-orderable-item)
())

(defclass pie (non-orderable-item)
())

(defclass donut (non-orderable-item)
())






;-----------------------------------------------------------------------
; Generic orderable-item class
; 
; The description gets printed out when the student asks what an item is.
; Subscenario is the name if the scenario that will be put on the stack
; if the item in ordered.  The subscenario will contains steps neccessary
; to find out an record additional information about an item.  For example
; would you like cream and sugar with your coffee?
; Subclasses are used to answer the question like "what kinds of X do
; you have?"
; The reference-name is a string that the waiter can use to remember what
; the student called something. For example, the student may have ordered
; "pepsi" which is a COLA object. Instead of calling it "COLA", it would
; be nice to call it "pepsi". 
;------------------------------------------------------------------------
(defclass orderable-item ()
  ((cost :accessor cost 
         :initform 0.00 
         :initarg :cost)
   (description :accessor description 
                :initform "An orderable item"
                :initarg :description)
    (subclasses :accessor subclasses
               :initform '("food" "drinks" "dessert")
               :initarg :subclasses)
   (reference-name :accessor reference-name
                   :initform ""
                   :initarg :reference-name)
      (subscenario :accessor subscenario
                 :initform nil
                 :initarg :subscenario)
   ))

;----------------
; Object methods
;----------------
(defmethod initialize ((self orderable-item))
;; This method isn't needed in this version of the objects file.  I just didn't
;; want to have to delete all of the calls.
;  (setf (parts self) (make-hash-table :size 7))
  self)



(defmethod print-object ((self orderable-item) stream)
  (format stream "~a" (type-of self)))

(defmethod dump-object ((self orderable-item) stream)
   (format stream "~%      ")
   (print-object self stream))


;-------------------------------
; the "nothing" object
;-------------------------------
(defclass nothing (orderable-item)
( (reference-name :initform "nothing")))                                

;-------------------------------
; drink-items
;------------------------------
(defclass drink-item (orderable-item)
  ((subclasses :initform '("hot drinks" "cold drinks"))
   ))

(defclass hot-drink (drink-item)
  ((subclasses :initform '("coffee" "tea" "hot-chocolate"))
   ))

(defclass coffee (hot-drink)
  ((cost :initform 1.25)
   (subclasses :initform '())
   (subscenario :initform coffee-scenario)
  (description 
   :initform "Coffee is a hot, bitter drink usually served with milk and sugar.")))

 
(defclass tea (hot-drink)
   ((cost :initform 1.25)  
    (subclasses :initform '())
   (description :initform 
       "Tea is a hot drink made from plant leaves. Black tea is from China.")))

(defclass hot-chocolate (hot-drink)
  ((cost :initform 1.25)
   (subclasses :initform '())
   (on-menu :initform nil)
   (description :initform
       "Hot chocolate is a milky, chocolate flavoured hot drink.")))
 
(defclass cold-drink (drink-item) 
  ((subclasses :initform '("soft-drink" "milk" "iced-tea" "water"))))


(defclass water (cold-drink)
((cost :initform 0.00)
 (subclasses :initform '())))

(defclass milk (cold-drink)
((cost :initform 1.00)
 (subclasses :initform '())))

(defclass iced-tea (cold-drink)
((cost :initform 1.25)
 (subclasses :initform '())))

(defclass soft-drink (cold-drink)
    ((cost :initform 1.25)
     (subscenario :initform pop-scenario)
     (subclasses :initform '("cola" "sprite" "orange-crush" "root-beer"))
      (description :initform "Pop is carbonated water with flavouring. We have cola, lemon-lime, and 
orange flavours.")))



(defclass cola (soft-drink)((subclasses :initform '())
))
(defclass sprite (soft-drink)((subclasses :initform '())
))
(defclass orange-crush (soft-drink)((subclasses :initform '())
))
(defclass root-beer (soft-drink)((subclasses :initform '())
))
      
;-------------------------------
; food
;-------------------------------
(defclass food-item (orderable-item)
 ((subclasses :initform '("fries" "burgers" "pasta" "steak"))))

(defclass pasta (food-item)
((subclasses :initform '("spaghetti" "ravioli"))
(cost :initform 0.0)))

(defclass steak (food-item)
((subclasses :initform '())
 (cost :initform 8.00)
 (subscenario :initform steak-scenario)))

(defclass ravioli (pasta)
((subclasses :initform '())
 (cost :initform 5.00)))
 
 (defclass spaghetti (pasta)
  ((subclasses :initform '())
   (cost :initform 5.00)))

(defclass fries (food-item)((subclasses :initform '())
                             (cost :initform 3.00)))
(defclass burger (food-item) ((subclasses :initform '("cheeseburger" "deluxe-burger" "hamburger"))))
                               
(defclass cheeseburger (burger) ((subclasses :initform '())
                                    (cost :initform 4.00)))

(defclass deluxe-burger (burger) ((subclasses :initform '())
                                 (cost :initform 4.50)))

(defclass hamburger (burger) ((subclasses :initform '())
                                  (cost :initform 3.50)))

(defclass dessert-item (orderable-item)
  ((subclasses :initform '("cake" "ice-cream"))))

(defclass cake (dessert-item)((subclasses :initform '("chocolate cake" "cheesecake"))
                             (cost :initform 1.50)))

(defclass chocolate-cake (cake)
((subclasses :initform '())
))

(defclass cheesecake (cake)
((subclasses :initform '())))


(defclass ice-cream (dessert-item) ((subclasses :initform '("vanilla ice-cream" "strawberry ice-cream"))
                                             (cost :initform 1.50)))

(defclass vanilla-ice-cream (ice-cream)
((subclasses :initform '())
))

(defclass strawberry-ice-cream (ice-cream)
((subclasses :initform '()))) 

;-------------------------------
; people
;-------------------------------

(defclass person ()((subclasses :initform '(customer))))
(defclass customer (person)((subclasses :initform '())))


























