;##########################################################################
;#                                                                        #
;#                            ORDER2.SCE                                  #
;#                                                                        #
;# CONTENTS:                                                              #
;#   parameters:                                                          #
;#     scenarios: dessert-order, drink-order, food-order, order-scenario  #
;#        steps:  ask-order, ask-again, ask-rephrased, accept-last-item   #
;#                accept-that-item, invalid-order, get-item, get-nothing  #
;#                do-you-have, is-served-here, how-about-do-you-have,     #
;#                what-about-kinds-of, kinds-of, describe-item, where-is, #
;#                tell-cost , ask-food-order, list-food-items,            #
;#                ask-drink-order, list-drink-items, ask-dessert-order    #
;#                list-drink-items                                        #
;##########################################################################
               
;------------------------------
; Order scenario
;------------------------------

(defparameter tell-cost 
  (make-instance 'mystep
;; tell-cost gives the cost of an orderable item and asks the user if they
;;would like to order that item
		 
		 :name "tell-cost"
		 :input-condition (orc 
				   (list 
				    (wordc 
				     :nec '(what cost (orderable-item focus))
				     :sup '(is))
				    (wordc 
				     :nec '(how much (orderable-item focus))
				     :sup '(is))))
		 :my-action #'(lambda (self restaurant)
				(let ((item 
				       (initialize 
				       (make-instance (obj-class (binding-value
								  self 'focus))
						      ))))
				  (say (waiter restaurant) 
				       "~%The price is $~a." (cost item))
				  (setf 
				   (expecting-input (waiter restaurant)) t))
				)))
                                           
(defparameter describe-item 
    (make-instance 'mystep
;; describe-item print the description of an orderable item and asks the
;; customer if they would like to order that item.

		   :name "describe-item"
		   :input-condition  (andc :nec 
					   (list
					    (wordc 
					     :nec '(what (orderable-item focus)
							 )
					     :sup '(is))
					    (notc (wordc :nec '(about)))
					    (notc (wordc :nec '(cost)))
					    (notc (wordc :nec '(kind-of)))))
                             
		   :my-action #'(lambda (self restaurant)
				  (say w "~%~a"  (description 
                                                  (get-binding 
                                                   (input-results self) 
                                                   'focus)))
				  ;; (say w "~%Would you like to order that?")
				  (setf (expecting-input (waiter restaurant)) t
					))))
 
(defparameter kinds-of 
  (make-instance 'mystep
;;kinds-of lists the subclasses of an orderable item
		 
		 :name "kinds-of"
		 :input-condition
                              
		 (wordc :nec '(what kind-of (orderable-item focus))
			:sup '( waiter have serve))
                
		 :my-action #'(lambda (self restaurant)
				(let* ((subclasses 
					(subclasses (make-instance 
						     (obj-class (binding-value
								 self 'focus)))
						    ))
				       (number-of-subclasses (length subclasses
								     )))
				  (say w "~%We have ")
				  (cond ((eq number-of-subclasses 1)
					 (say w "just ~a." 
					      (name (car subclasses ))))
					((eq number-of-subclasses 0)
					 (say w 
					      "only one kind of ~a."  
					      (string-downcase 
					       (form-in-text 
						(binding-value self 'focus)))))
					(t (loop for  i from 0 to 
						 (- number-of-subclasses 2) do
						 (say w "~a, " 
						      (nth i subclasses)))
					   (say w "and ~a."
						(nth (- number-of-subclasses 1)
						     subclasses))))
                                  (setf (expecting-input (waiter restaurant)) t
					)))))

(defparameter what-about-kinds-of 
  (make-instance 'mystep
;;this step is used as a follow up to kinds-of

		 :name "what-about-kinds-of"
		 :input-condition 
		 (wordc :nec '(about (orderable-item focus))
			:sup '(how what))
                                 
		 :my-condition (lambdac #'(lambda (restaurant mystep)
					    (prev-step-matches 
					     restaurant 
					     '("kinds-of" "what-about-kinds-of"
					       ) 4)))
            
		 :my-action #'(lambda (self restaurant)
				(let* ((subclasses 
					(subclasses (make-instance 
						     (obj-class (binding-value
								 self 'focus)))
						    ))
				       (number-of-subclasses 
					(length subclasses)))
				  
				  (say w "~%We have ")
				  (cond ((eq number-of-subclasses 1)
					 (say w "just ~a." 
					      (name (car subclasses ))))
					((eq number-of-subclasses 0)
					 (say w 
					      "only one kind of ~a."  
					      (string-downcase 
					       (form-in-text 
						(binding-value self 'focus)))))
					(t (loop for  i from 0 to 
						 (- number-of-subclasses 2) do
						 (say w "~a, " 
						      (nth i subclasses)))
					   (say w "and ~a."
						(nth (- number-of-subclasses 1)
						     subclasses))))
                                  (setf (expecting-input (waiter restaurant)) t
					)))))



(defparameter do-you-have 
  (make-instance 'mystep
;;do-you-have answers yes if the item is an orderable item.  If it is, the 
;;customer is then asked if they would like to order that item.
		 
		 :name "do-you-have"
		 :input-condition (orc (list 
					(wordc :nec '(waiter serve)
					       :sup '((orderable-item focus)))
					(wordc :nec '(waiter have)
					       :sup '((orderable-item focus))))
				       )

		 :my-action 
		 #'(lambda 
		     (self restaurant)
		     (if (binding-value self 'focus)
			 (format t 
				 "~%Yes we do.  Would you like to order that?")
		       (format t "~%Sorry we don't"))
		     (setf (expecting-input (waiter restaurant)) t))))


(defparameter is-served-here 
  (make-instance 'mystep
;;do-you-have answers yes if the item is an orderable item.  If it is, the 
;;customer is then asked if they would like to order that item.

		 :name "is-served-here"
		 :input-condition 
		 (wordc :nec '(be (orderable-item focus) serve here))

		 :my-action 
		 #'(lambda (self restaurant)
		     (if (binding-value self 'focus)
			 (format t "~%Yes.  Would you like to order that?")
		       (format t "~%Sorry, no."))
		     (setf (expecting-input (waiter restaurant)) t))))

(defparameter how-about-do-you-have 
  (make-instance 'mystep
 ;; this is a follow-up step to do-you-have

		 :name "how-about-do-you-have"
		 :input-condition (orc (list 
					(wordc :nec '(what about)
					       :sup '((orderable-item focus)))
					(wordc :nec '(how about)
					       :sup '((orderable-item focus))))
				       )
                                
		 :my-condition (lambdac #'(lambda (restaurant mystep)
					    (prev-step-matches 
					     restaurant 
					     '("do-you-have" 
					       "how-about-do-you-have" 
					       "is-served-here") 4)))
            
		 :my-action 
		 #'(lambda (self restaurant)
		     (if (binding-value self 'focus)
			 (format t "~%Yes we do.  Would you like to get that?")
		       (format t "~%Sorry we don't"))
		     (setf (expecting-input (waiter restaurant)) t))))       
              


(defparameter invalid-order 
  (make-instance 'mystep
;;This step is executed when the customer tries to order something that could
;; be ordered in a restaurant, but is not available here.
		 
		 :name "invalid-order"
		 :input-condition (orc 
				   (list 
                                    (phrasec
				     (list
				      anystring :nec 'customer 
				      (wildcard 0 2)
				      '(want have order) 
				      :nec '(non-orderable-item)))
                                    (phrasec (list
					      anystring 
					      :nec '(non-orderable-item)))
				    )) 
		 :my-condition   (orc (list
				       (spscorec :sc 1 :nec 
						 (wordc :nec 
							'(non-orderable-item) 
							) )
				       (lambdac #'(lambda 
						    (restaurant mystep)
						    (prev-step-matches
                                                     restaurant 
						     '("ask-order" 
						       "ask-food-order" 
						       "ask-drink-order" 
						       "ask-dessert-order") 
						     1)))))
                 :my-action #'(lambda (self restaurant)
				(let ((context 
				       (name (current-scenario 
					      (waiter restaurant))))
				      (items '())
				      (non-orderable-items '())
				      (temp '())) 
				  (loop for word in 
					(input (waiter restaurant)) do 
					(if 
					    (eq (class-name (class-of word)) 
						'noun)
					    (push word items)))
				  
				  (loop for word in items do
					(setf temp 
					      (make-instance (base-form word)))
					(if (kind-of 
					     (class-name (class-of temp)) 
					     'non-orderable-item)
					    (push word non-orderable-items)))
				  
				  
				  (say w "~%Sorry, we don't serve ")
				  (cond ((eq (length non-orderable-items) 1)
					 (say w "~a" 
					      (form-in-text 
					       (car non-orderable-items))))
					(t
					 (loop for i from 0 to 
					       (- (length non-orderable-items)
						  2) do
						  (say w "~a, " 
						       (form-in-text 
							(nth i 
							    non-orderable-items
							    ))))
					 (say w "or ~a" 
					      (form-in-text 
					       (nth (- 
						     (length 
						      non-orderable-items) 1) 
						    non-orderable-items)))))
				  (say w " here.")
				  (say w "~%Can I bring you something else")
				  (cond ((equal context "drink-order")
					 (say w " to drink?"))
					((equal context "food-order")
					 (say w " to eat?"))
					((equal context "dessert-order")
					 (say w " for dessert?"))
					(t (say w "?")))
				  (setf (expecting-input (waiter restaurant)) t
					))
				)
		 ))

                                 

(defparameter get-item 
  (make-instance 'mystep
;;This step is executed when something that resembles an order is placed.
;;The item is written on the order pad and a slot scenario is made and placed
;; on top the the waiter scenario stack.  If the item has subclasses, an 
;;attributes scenario is also made and placed on top of the stack.

		 :name "get-item"
		 :input-condition (orc 
				   (list 
                                    (phrasec
				     (list
				      anystring :nec 'customer 
				      (wildcard 0 2)
				      '(want have order) 
				      :nec focus-np-cond))
                                    (phrasec (list
					      anystring :nec focus-np-cond))
				    )) 
		 :my-condition  (andc :nec 
				      (list 
				       (orc 
					(list
					 (spscorec :sc 1 :nec 
						   (wordc :nec 
							  '(orderable-item) ) )
					 (lambdac #'(lambda 
						      (restaurant mystep)
						      (prev-step-matches
						       restaurant 
						       '("ask-order" 
							 "ask-food-order" 
							 "ask-drink-order" 
							 "invalid-order" 
							 "ask-dessert-order" 
							 "washroom") 
						       2)))))
				       (lambdac 
					'(lambda (rest mystep)
					   (let 
					       ((det-val 
						 (binding-value mystep 
								'determiner)))
					     (if 
						 (and det-val 
						      (definite det-val))
						 (make-instance 'results 
								:success nil)
					       (make-instance 'results 
							      :success t 
							      :score 1)))))))
                                                                  
		 
		 :my-action #'(lambda (self restaurant)
				(let 
				    ((items-to-add (binding-values self 
								   'focus))
				     (current-item '())
				     (item-names '())
				     (print-string ""))
				;;(format t "~%items-to-add: ~a" items-to-add)
                               
				  (loop for item in items-to-add do
					(setf current-item (initialize 
							    (make-instance 
							     (obj-class item)))
					      )
					(setf (reference-name current-item) 
					      (string-downcase 
					       (form-in-text item)))
					(add-to-order-pad 
					 (order-pad restaurant) current-item)
					(increment-counters 
					 (order-pad restaurant) current-item)
					

		 ;;This is to make things easier in the serve scenarios.
					(cond 
					 ((kind-of 
					   (class-name (class-of current-item)
						      ) 'drink-item)
					   (delete 'nothing 
						   (drink-items 
						    (order-pad restaurant))))
					  ((kind-of 
					    (class-name (class-of current-item)
							) 'dessert-item)
                         
					   (delete 'nothing 
						   (dessert-items 
						    (order-pad restaurant))))
					  (t (delete 'nothing 
						     (main-course 
						      (order-pad restaurant))))
					  )
                           
					(cond 
					 ((null (subclasses current-item))
					  (cond 
					   (
					    (not (eq (subscenario current-item)
						     nil))
					    (setf 
					     (item (subscenario current-item))
					     current-item) 
					    (push (subscenario current-item) 
						  (scenario-stack 
						   (waiter restaurant))))))
                            
					 
					 (t (make-attributes-scenario 
					     current-item restaurant)
                          
					    ))

					(push 
					 (copy-seq (reference-name current-item
								   )) 
					 item-names))
                      
				  (setf item-names (reverse item-names))
				  (setf print-string (nstring-upcase 
						      (pop item-names) 
						      :start 0 :end 
						      1))
                               
				  (loop for i from 0 below 
					(- (length item-names) 1) do
					(setf print-string 
					      (format nil 
						      "~a, ~a" print-string 
						      (nth i item-names))))

				  (when item-names 
					(setf print-string 
					      (format nil "~a and ~a" 
						      print-string 
						      (car (last item-names))))
					)
                                
				  (say w "~%~a~a" 
				       (random-pick 
					(list (format nil "~a? " print-string)
					      )) (random-pick sure) )
				  
				  ))))

                                                                        

(defparameter get-nothing 
  (make-instance 'mystep
;;This step is executed when the customer does not want to order anything.

		 :name "get-nothing"
		 :input-condition 
		 (andc :nec (list
			     (notc (wordc :nec '(with)))
			     (orc  (list (wordc :nec '(nothing) 
						:sup '(thanks more))
					 (wordc :nec '(no) :sup '(thanks))))))
		 :my-condition   (lambdac 
				  '(lambda (rest mystep)
				     (prev-step-matches rest 
							'("ask-food-order" 
							  "ask-drink-order" 
							  "ask-order" 
							  "ask-dessert-order" 
							  "invalid-order" 
							  "list-drink-items" 
							  "list-food-items" 
							  "list-dessert-items"
							  ) 3)))
                              
		 :my-action #'(lambda (self rest)
				(let ((context 
				       (name (current-scenario (waiter rest))))
				      )
				      (cond ((equal context "drink-order")
					     (say w "~%Nothing to drink? ~a" 
						  (random-pick sure))
					     (add-to-section (order-pad rest) 
							     "drink-items" 
							     (make-instance 
							      'nothing)))
                                       
					    ((equal context "dessert-order")
					     (say w "~%Nothing for dessert? ~a"
						  (random-pick sure))
					     (add-to-section (order-pad rest) 
							     "dessert-items" 
							     (make-instance 
							      'nothing)))
					    ((equal context "food-order")
					     (say w "~%Nothing to eat? ~a" 
						  (random-pick sure))
					     (add-to-section (order-pad rest) 
							     "main-course"
							     (make-instance 
							      'nothing)))
					    (t (say w "Okay...")))
				      
				      ))))
  
(defparameter accept-last-item 
  (make-instance 'mystep
;;accept-last-item is executed after the customer answers yes to the 
;;step "do-you-have" or to the step "describe item".
;;the last item is the written on the order-pad and the attributes scenario
;;is created if needed.  Also, the slot scenario is created.

		 :name "accept-last-item"
		 :input-condition  (wordc :nec '(yes))
		 :my-condition  (lambdac 
				 #'(lambda (rest mystep)
				     (let 
					 ((results 
					   (prev-step-matches 
					    rest 
					    '("do-you-have"
					      "how-about-do-you-have"  
					      "is-served-here") 4))
					  (prev-step (last-step (waiter rest)))
					  )
				       (when (success results)
					     (setf (bindings results)
						   (bindings 
						    (input-results prev-step)))
					     (setf (bindings results) 
						   (append 
						    (bindings 
						     (results prev-step))
						    (bindings results))))
				       results)))
					
		 :my-action #'(lambda (self restaurant)
				(let ((item-to-add 
				       (initialize (make-instance 
						    (obj-class 
						     (binding-value self 
								    'focus)))))
				      )
				  (setf (reference-name item-to-add)
					(string-downcase (form-in-text 
							  (binding-value self 
									 'focus
									 ))))
				  (add-to-order-pad (order-pad restaurant) 
						    item-to-add)
				  (increment-counters (order-pad restaurant) 
						      item-to-add)

;;This is to make things easier in the serve scenarios.

				  (cond ((kind-of 
					  (class-name (class-of item-to-add)) 
					  'drink-item)
					 (delete 'nothing (drink-items 
							   (order-pad 
							    restaurant))))
					((kind-of (class-name (class-of 
							       item-to-add)) 
						  'dessert-item)
					 (delete 'nothing (dessert-items 
							   (order-pad 
							    restaurant))))
					(t (delete 'nothing (main-course 
							     (order-pad 
							      restaurant)))))
                          
				  (cond ((null (subclasses item-to-add))
					 (cond 
					  ((not (eq (subscenario item-to-add) 
						    nil))
					   (setf (item (subscenario 
							item-to-add)) 
						 item-to-add) 
                             
					   (push (subscenario item-to-add) 
						 (scenario-stack 
						  (waiter restaurant))))))
                                 
					(t 
					 (make-attributes-scenario item-to-add
								   restaurant)
					 ))
                     
			 ;(format t "~%~a? Sure." 
		   ;        (base-form (binding-value previous-step 'focus)))
				  (say w "~%~a" (random-pick sure))
                        
				  ))))

(defparameter accept-that-item 
  (make-instance 'mystep
;;accept-last-item is executed after the customer answers yes to the 
;;step "do-you-have" or to the step "describe item".
;;the last item is the written on the order-pad and the attributes scenario
;;is created if needed.  Also, the slot scenario is created.

		 :name "accept-that-item"
		 :input-condition  
		 (wordc :nec '(that)
			:sup '(customer have))
		 :my-condition  
		 (lambdac #'(lambda (rest mystep)
			      (let ((results 
				     (prev-step-matches rest 
							'( "describe-item" 
							   "tell-cost" 
							   "kinds-of") 4))
				    (prev-step (last-step (waiter rest))))
				(when (success results)
				      (setf (bindings results)
					    (bindings (input-results prev-step)
						      ))
				      (setf (bindings results) 
					    (append (bindings 
						     (results prev-step))
						    (bindings results))))
				results)))
		 
		 :my-action #'(lambda (self restaurant)
				(let ((item-to-add (initialize (make-instance 
								(obj-class 
								 (binding-value
								  self 'focus))
								))))
				  (setf (reference-name item-to-add)
					(string-downcase 
					 (form-in-text (binding-value self 
								      'focus)))
					)
				  (add-to-order-pad (order-pad restaurant) 
						    item-to-add)
				  (increment-counters (order-pad restaurant) 
						      item-to-add)

;;This is to make things easier in the serve scenarios.

				  (cond ((kind-of (class-name 
						   (class-of item-to-add)) 
						  'drink-item)
					 (delete 'nothing (drink-items 
							   (order-pad 
							    restaurant))))
					((kind-of (class-name 
						   (class-of item-to-add)) 
						  'dessert-item)
					 (delete 'nothing (dessert-items 
							   (order-pad 
							    restaurant))))
					(t (delete 'nothing (main-course 
							     (order-pad 
							      restaurant)))))
                          
				  (cond ((null (subclasses item-to-add))
					 (cond ((not (eq (subscenario 
							  item-to-add) nil))
						(setf (item (subscenario 
							     item-to-add)) 
						      item-to-add)
						
						(push (subscenario item-to-add)
						      (scenario-stack 
						       (waiter restaurant))))))
                                 
					(t (make-attributes-scenario 
					    item-to-add restaurant)
					   ))
                     
                         ;(format t "~%~a? Sure." 
                         ;    (base-form (binding-value previous-step 'focus)))
				  (say w "~%~a" (random-pick sure))
				  ))))

(defparameter ask-order 
  (make-instance 'mystep
		 :name "ask-order"
		 :my-condition (andc :nec 
				     (list 
				      (notc 
				       (orc (list 
					     (historyc "ask-order") 
					     (historyc "ask-drink-order") 
					     (historyc "ask-food-order")
					     (historyc "ask-dessert-order")
					     (historyc "get-order"))))
				      (historyc "deliver-menu")))
		 :my-action #'(lambda (self restaurant)
				(say w "~%Are you ready to order now?")
				(setf (expecting-input (waiter restaurant)) t))
		 ))
  
(defparameter ask-again 
  (make-instance 'mystep
		 :name "ask-again"
		 :input-condition (wordc :nec '(later))
		 :my-condition  (lambdac 
				 #'(lambda (rest mystep)
				     (prev-step-matches rest 
							'("ask-drink-order" 
							  "ask-again" 
							  "ask-dessert-order" 
							  "ask-food-order") 1))
				 )
                       
		 :my-action 
		 #'(lambda (self restaurant)
		     (say w 
			 "~%Okay, I'll be back in a minute to take your order."
			 )
		     (wait 5)
		     (say w "~%Now are you ready to order?")
		     (setf (expecting-input (waiter restaurant)) t))))

(defparameter ask-rephrased 
  (make-instance 'mystep
		 :name "ask-order"
		 :input-condition (wordc :nec '(yes))
		 :my-condition  (lambdac #'(lambda (rest mystep)
					     (prev-step-matches 
					      rest '("ask-order" "ask-again" 
						     "ask-drink-order"  
						     "ask-food-order" 
						     "ask-dessert-order" 
						     "invalid-order")
					      2)))
		 :my-action #'(lambda (self restaurant)
				(say w "~%What can I get for you?")
				(setf (expecting-input (waiter restaurant)) t)
)))

(defparameter where-is 
  (make-instance 'mystep
		 :name "where-is"
		 :input-condition 
		 (phrasec 
		  (list anystring :nec 'where 'be :nec focus-np-def-det 
			(wildcard 0 10) 'question-mark))                     
		 :my-action #'(lambda (self restaurant)
				(say w "~%Your ~a is coming. (Sorry!)" 
				     (form-in-text (binding-value self 'focus))
				     )
                      ; (setf (expecting-input (waiter restaurant)) t)
				) ))


(defparameter order-scenario 
  (make-instance 'scenario
		 :name "order-scenario"
		 :parent general-scenario
		 :end-condition (orc (list (historyc  "get-item")
					   (historyc  "accept-last-item")
					   (historyc "accept-that-item")))))
(add-steps order-scenario 
	   (list ask-order 
		 ask-again
		 ask-rephrased
		 accept-last-item
		 accept-that-item
		 invalid-order 
		 get-item 
		 get-nothing
		 do-you-have
		 is-served-here 
		 how-about-do-you-have
		 what-about-kinds-of
		 kinds-of 
		 describe-item
		 where-is
		 tell-cost))

  ;(format t "Finished building order-scenario~%")            
                   
;------------------------------
; Food-order scenario
;------------------------------
(defparameter ask-food-order 
  (make-instance 'mystep
		 :name "ask-food-order"
		 :my-condition
                 (andc :nec (list
                             (notc (historyc "present-bill")) 
			     (notc (lambdac #'(lambda (restaurant mystep)
						(prev-step-matches 
						 restaurant 
						 '("ask-food-order") 2))))
			     (notc (lambdac #'(lambda (restaurant mystep)
						(step-appears restaurant 
							      "ask-food-order"
							      2 2))))
			     (lambdac #'(lambda (restaurant mystep)
					  (if 
					      (or (main-course 
						   (order-pad restaurant))
						  (dessert-items 
						   (order-pad restaurant)))
					      (make-instance 'results
							     :success nil)
					    (make-instance 'results
							   :success t
							   :score 1))))
			     (notc (spscorec :sc 1 :nec 
					     (wordc :nec '(food-item))))))
		 :my-action #'(lambda (self restaurant)
				(say w "~%~a" (random-pick food-question))
				(setf (expecting-input (waiter restaurant)) t))
		 ))

(defparameter list-food-items 
  (make-instance 'mystep
		 :name "list-food-items"
		 :input-condition (andc :nec 
					(list (notc (wordc :nec '(about)))
					      (orc (list
						    (wordc 
						     :nec '(what waiter have))
						    (wordc 
						     :nec '(what waiter serve))
						    (wordc 
						     :nec '(what can customer 
								 have))  
						    ))))                 
                 
		 :my-action 
		 #'(lambda 
		     (self restaurant)
		     (print-food restaurant)
		     (say 
		      (waiter restaurant) 
       "~%We also have a few other main-course items available upon request.")
		     (setf (expecting-input (waiter restaurant)) t) ) ) )
             


(defparameter food-order 
  (make-instance 'scenario
		 :name "food-order"
		 :description "Get something to eat."
		 :my-condition (orc (list (historyc "drink-order")
					  (spscorec :sc 1 :nec 
						    (wordc 
						     :nec '((food-item focus)))
						    )
					  (lambdac 
					   '(lambda (rest mystep) 
					      (cond ((member 'drink-order 
							     (scenario-stack 
							      (waiter rest)))
						     (make-instance 
						      'results 
						      :success nil))
						    (t (make-instance 
							'results 
							:success t)))))
					  ))
		 :parent order-scenario
		 :end-condition  (orc (list (historyc "present-bill")
					    (lambdac #'(lambda (restaurant 
								mystep)
							 (if (or (main-course 
								  (order-pad 
								   restaurant))
								 (dessert-items
								  (order-pad 
								   restaurant))
								 )
							     (make-instance 
							      'results 
							      :success t)
							   (make-instance 
							    'results 
							    :success nil))))))
		 ))
(add-steps food-order (list 
		       ask-food-order 
		       list-food-items
		       ))
  
  ;(format t "Finished building food-order scenario~%")

;------------------------------
;  Drink-order scenario
;------------------------------
(defparameter ask-drink-order 
  (make-instance 'mystep
		 :name "ask-drink-order"
		 :my-condition
                 (andc :nec (list 
                             (notc (lambdac #'(lambda (restaurant mystep)
						(prev-step-matches 
						 restaurant 
						 '( "ask-drink-order") 2))))
                             (notc (lambdac #'(lambda (restaurant mystep)
						(step-appears 
						 restaurant "ask-drink-order" 2
						 2))))
                             (lambdac #'(lambda (restaurant mystep)
					  (if (drink-items 
					       (order-pad restaurant))
					      (make-instance 'results
							     :success nil)
					    (make-instance 'results
							   :success t
							   :score 0))))
			     (notc (spscorec :sc 1 :nec 
					     (wordc 
					      :nec '(food-item 
						     dessert-item 
						     non-orderable-item))))))
		 :my-action #'(lambda (self restaurant)
				(say w "~%~a" (random-pick drink-question))
				(setf (expecting-input (waiter restaurant)) t))
		 ))


(defparameter list-drink-items 
  (make-instance 'mystep
		 :name "list-drink-items"
		 :input-condition (andc :nec 
					(list (notc (wordc :nec '(about)))
					      (orc 
					       (list (wordc 
						      :nec '(what waiter serve)
						      )     
						     (wordc 
						      :nec '(what waiter have))
						     (wordc 
						      :nec '(what can customer
								  have))
						     ))))                  
		 
		 :my-action 
		 #'(lambda (self restaurant)
		     (print-drinks restaurant)
		     (say (waiter restaurant) 
	     "~%We also have a number of other drinks available upon request.")
		     (setf (expecting-input (waiter restaurant)) t) ) ) )
             


  
(defparameter drink-order 
  (make-instance 'scenario
		 :name "drink-order"
		 :description "Order yourself a coffee or something nice."
		 :my-condition (historyc "simple greeting scenario")
		 :parent order-scenario
		 :end-condition (orc (list (historyc "present-bill")
					   (lambdac 
					    #'(lambda (restaurant mystep)
						(if (drink-items 
						     (order-pad restaurant))
						    (make-instance 
						     'results :success t)
						  (make-instance 
						   'results :success nil))))))
		 ))
                              
(add-steps drink-order (list 
			ask-drink-order list-drink-items
			))

 ; (format t "Finished building drink-order scenario~%")

;------------------------------
;  Dessert-order scenario
;------------------------------

(defparameter ask-dessert-order 
  (make-instance 'mystep
		 :name "ask-dessert-order"
		 :my-condition
                 (andc :nec (list 
                             (notc (historyc "present-bill"))
			     
                             (notc (lambdac #'(lambda (restaurant mystep)
						(prev-step-matches 
						 restaurant 
						 '("ask-dessert-order") 2))))
                             (notc (lambdac #'(lambda (restaurant mystep)
						(step-appears 
						 restaurant "ask-dessert-order"
						 2 2))))
                             (lambdac #'(lambda (restaurant mystep)
					  (if (dessert-items 
					       (order-pad restaurant))
					      (make-instance 'results
							     :success nil)
					    (make-instance 'results
							   :success t
							   :score 0))))
			     (notc (spscorec :sc 1 :nec (wordc 
							 :nec 
							 '(dessert-item 
							   non-orderable-item))
					     ))))
		 :my-action #'(lambda (self restaurant)
				(say w "~%~a" (random-pick dessert-question))
				(setf (expecting-input (waiter restaurant)) t))
		 ))


(defparameter list-dessert-items 
  (make-instance 'mystep
		 :name "list-dessert-items"
		 :input-condition 
		 (andc :nec (list (notc (wordc :nec '(about)))
				  (orc (list (wordc :nec '(what waiter serve))
					     (wordc :nec '(what waiter have)
						    )
					     (wordc 
					      :nec '(what can customer have))))
				  ))                                  
		 :my-action #'(lambda (self restaurant)
				(print-dessert restaurant)
				(setf (expecting-input (waiter restaurant)) t)
				) ) )
             



(defparameter dessert-order 
  (make-instance 'scenario
		 :name "dessert-order"
		 :description "Order dessert."
		 :my-condition (orc (list (historyc "food-order")
					  (spscorec :sc 1 :nec 
						    (wordc 
						     :nec '((dessert-item focus
									  ))))
					  (lambdac '(lambda (rest mystep)
						      (cond ((member 
							      'dessert-order
							      (scenario-stack 
							       (waiter rest)))
							     ( make-instance 
							       'results 
							       :success nil))
							    (t (make-instance 
								'results 
								:success t)))))
					  ))
		 :end-condition (orc (list (historyc "present-bill")
					   (lambdac 
					    #'(lambda (restaurant mystep)
						(if (dessert-items 
						     (order-pad restaurant))
						    (make-instance 'results 
								   :success t)
						  (make-instance 'results 
								 :success nil))
						))))
		 :parent order-scenario))
(add-steps dessert-order (list ask-dessert-order list-dessert-items))
















