;###################################################################
;#                                                                 #
;#                    RESTAURA.LSP                                 #
;#                                                                 #
;#       CONTENTS:                                                 #
;#           Classes:  restaurant                                  #
;#           Parameters:  transcript                               #
;#                                                                 #
;#           Comments:  This file contains the restaurant class    #
;#                      and its associated methods.                #
;#                                                                 #
;###################################################################


;***************************************************
;*                                                 *
;*           RESTAURANT CLASS                      *
;*                                                 *
;*  This class contains an instance of the waiter  *
;* class, the order-form class, a table and the    *
;* menu.                                           *
;***************************************************

(defclass restaurant ()
     ((waiter 
        :initarg :waiter
        :initform (make-instance 'waiter)
        :accessor waiter)
      (order-pad
       :initarg :order-pad
       :initform (make-instance 'order-form)
       :accessor order-pad)
      (table
       :initarg :table
       :initform (make-instance 'order-form)
       :accessor table)
      (menu 
       :initarg :menu
       :accessor menu
       :initform '((drinks soft-drinks coffee tea) (food fries pasta steak) (dessert cake ice-cream))) ))

(defmethod print-object ((self restaurant) stream)
  (format stream "<the restaurant>"))

(defmethod print-drinks ((self restaurant))
;; Prints the drinks available on the menu.  The is done in the step
;; list-drink-items.

  (let ((group (first (menu self))))
  (say (waiter self) "~%For ~a we have " (string-downcase (symbol-name (first
         group))))
   (say (waiter self) "~a" (string-downcase (symbol-name (second group))))
    (when (> (length group) 3)
        (loop for index from 2 below (-(length group) 1) do
          (say (waiter self) ", ~a" (string-downcase (symbol-name (nth index group))))))
    (when (>(length group) 2)
     (say (waiter self) " and ~a." (string-downcase (symbol-name (car (last group))))))) )


(defmethod print-dessert ((self restaurant))
;; Prints the desserts on the menu.  This is done in the list-dessert-items
;; step.
(let ((group (third (menu self))))
  (say (waiter self) "~%For ~a we have " (string-downcase (symbol-name (first
         group))))
   (say (waiter self) "~a" (string-downcase (symbol-name (second group))))
    (when (> (length group) 3)
        (loop for index from 2 below (-(length group) 1) do
          (say (waiter self) ", ~a" (string-downcase (symbol-name (nth index group))))))
    (when (>(length group) 2)
     (say (waiter self) " and ~a." (string-downcase (symbol-name (car (last group))))))) )


(defmethod print-food ((self restaurant))
;; Prints the food-items on the menu.  This is done in the list-food-items
;; step.
  (let ((group (second (menu self))))
  (say (waiter self) "~%For ~a we have " (string-downcase (symbol-name (first
         group))))
   (say (waiter self) "~a" (string-downcase (symbol-name (second group))))
    (when (> (length group) 3)
        (loop for index from 2 below (-(length group) 1) do
          (say (waiter self) ", ~a" (string-downcase (symbol-name (nth index group))))))
    (when (>(length group) 2)
     (say (waiter self) " and ~a." (string-downcase (symbol-name (car (last group))))))) )

(defmethod print-menu ((self restaurant))
;; Prints the entire menu.
  (loop for group in (menu self) do
        (say (waiter self) "~%   for ~a we have " (string-downcase (symbol-name (first group))))
	(say (waiter self) "~a" (string-downcase (symbol-name (second group))))
        (when (> (length group) 3)
	  (loop for index from 2 below (- (length group) 1) do
		(say (waiter self) ", ~a" (string-downcase (symbol-name (nth index group))))))
        (when  (> (length group) 2) 
	  (say (waiter self) " and ~a" (string-downcase (symbol-name (car (last group))))))))

(defparameter transcript '())
                  
(defmethod run ((self restaurant) )
;; The method invokes the do-likeliest-scenario method in order to start
;; the restaurant simulation.
   
   (with-open-file (transcript output-file :direction :output :if-exists :append) 
      (format transcript "~%~%")
      (setf *TRAP* 0)
      (do-likeliest-scenario (waiter self) self) ))











