;######################################################################
;#                                                                    #
;#                     RUN.SCE                                        #
;#                                                                    #
;#  CONTENTS:                                                         #
;#    Parameters:  start-dialogue                                     #
;#                    steps:  run-dialogue-system, do-not-run,        #
;#                            are-you-ready                           #
;#                post-dialogue-analysis                              #
;#                    steps:  ask-if-post, yes-post, no-post          #
;#                           by-category, by-row                      #
;######################################################################      



(defparameter are-you-ready 
  (make-instance 'mystep
		 :name "are-you-ready"
		 :my-condition (spscorec :sc 2 
					 :nec (notc 
					       (historyc "are-you-ready")))
		 :my-action 
		 #'(lambda (self restaurant)
		     (say (waiter restaurant) 
			  "~%Are you ready to run the dialogue system?")
		     (setf (expecting-input (waiter restaurant)) t))))


(defparameter run-dialogue-system 
  (make-instance 'mystep
		 :name "run-dialogue-system"
		 :input-condition (wordc :nec '(yes))
		 :my-condition (historyc "are-you-ready")
		 :my-action #'(lambda (self restaurant)
				(setf (input (waiter restaurant)) '())
				(setf (scenario-stack (waiter restaurant)) 
				      (list 
				       start-dialogue greeting drink-order 
				       serve food-order  serve dessert-order 
				       serve end-scenario serve wrap-up 
				       post-dialogue-analysis)) 
				(format t "~%~%~%")

(format t "~%***********************************************************")
                      
                     (say (waiter restaurant)
"~%Welcome to Susie's restaurant! This is a simulation of a restaurant ~%~a~%~a~%~a~%~a~%~a~%~%"
   "where you can practice your English skills. At Susie's, the menu is"
   "very limited.  There are, however, many more items available. At"
   "Susie's you can ask whether your favourite food is served here, and"
   "do a few simple things that are commonly done in restaurants.  So, "
   "feel free to experiment."
    "Come on in!"  )



 

(format t "~%")
)))



(defparameter do-not-run 
  (make-instance 'mystep
           :name "do-not-run"
           :input-condition (notc (wordc :nec '(yes)))
           :my-condition (historyc "are-you-ready")
           :my-action #'(lambda (self restaurant)
			  (say (waiter restaurant) 
			       "~%Maybe some other time."))))


(defparameter start-dialogue 
  (make-instance 'scenario
		 :name "start-dialogue"
		 :steps (list run-dialogue-system do-not-run are-you-ready)
		 :end-condition (orc (list
				      (historyc "run-dialogue-system")
				      (historyc "do-not-run")))))




(defparameter ask-if-post-dialogue 
  (make-instance 'mystep
		 :name "ask-if-post-dialogue"
		 :my-condition (spscorec :sc 2 :nec 
					 (notc 
					  (historyc "ask-if-post-dialogue")))
		 :my-action #'(lambda (self restaurant)
				(format t "~%")

         (format t "~%***********************************************************")

         (format t "~%Would you like to see the post dialogue analysis?")
         (setf (expecting-input (waiter restaurant)) t))))


(defparameter no-post 
  (make-instance 'mystep
		 :name "no-post"
		 :input-condition (wordc :nec '(no))
		 :my-condition (lambdac #' (lambda (restaurant mystep)
					     (prev-step-matches 
					      restaurant 
					      '("ask-if-post-dialogue") 3)))
		 :my-action #'(lambda (self restaurant)
				(format t "~%Fine."))))

(defparameter yes-post 
  (make-instance 'mystep
		 :name "yes-post"
		 :input-condition (wordc :nec '(yes))
		 :my-condition (lambdac #' (lambda (restaurant mystep)
					     (prev-step-matches 
					      restaurant 
					      '("ask-if-post-dialogue") 3)))
		 :my-action 
		 #'(lambda (self restaurant)
		     (format t 
  "~%Would you like the errors grouped into input line numbers or categories?")
		     (setf (expecting-input (waiter restaurant)) t))))


(defparameter by-category 
  (make-instance 'mystep
		 :name "by-category"
		 :input-condition (wordc :nec '(category))
		 :my-action #'(lambda (self restaurant) 
				(print-errors-by-category list-of-errors))))

(defparameter by-row 
  (make-instance 'mystep
		 :name "by-row"
		 :input-condition (notc (wordc :nec '(category)))
		 :my-condition 
		 (lambdac #'(lambda (restaurant mystep)
			      (prev-step-matches restaurant '("yes-post") 2)))
		 :my-action #'(lambda (self restaurant)
				(print-by-line-number list-of-errors))))

(defparameter post-dialogue-analysis 
  (make-instance 'scenario
		 :name "post-dialogue-analysis"
		 :parent nil
		 :steps (list ask-if-post-dialogue no-post yes-post 
			      by-category by-row)
		 :end-condition (orc (list
				      (historyc "no-post")
				      (historyc "by-category")
				      (historyc "by-row")))))
 





























