;***************************************************************************
;*                                                                         *
;*                      NON-DICTIONARY CLASS                               *
;*                                                                         *
;*  This class is similar to the dictionary class, however, this class     *
;* will hold possible mispellings of keywords in the regular dictionary    *
;*                                                                         *
;***************************************************************************

(defclass non-dictionary ()
  ((contents
    :initarg :contents
    :accessor contents
    :initform '())))


;***************************************************************************
;*                                                                         *
;*                      WRONG-WORD CLASS                                   *
;*                                                                         *
;*  This class will be used to hold individual word in the non-dictionary. *
;* Each word has a place to store the correct spelling, a list of possible *
;* mispellings and the actual spelling given by the user.                  *
;*                                                                         *
;***************************************************************************


(defclass wrong-word ()
  ( (correct-form
     :initarg :correct-form
     :accessor correct-form
     :initform "")
    (other-forms
     :initarg :other-forms
     :accessor other-forms
     :initform '())
    (form-given
     :initarg :form-given
     :accessor form-given
     :initform  "")))



;**************************************************************************
;*                                                                        *
;*                  SPELLING-MISTAKE CLASS                                *
;*                                                                        *
;*  Instances of this class hold information about individual spelling    *
;* mistakes that the user has made.  It contains the correct spelling,    *
;* the spelling the user gave and the line on which the error was made.   *
;*                                                                        *
;**************************************************************************



(defclass spelling-mistake ()
  ((input-line-num 
    :initarg :line-num
    :accessor line-num
    :initform "")
   (mistake 
    :initarg :mistake
    :accessor mistake
    :initform "")
   (correction
    :initarg :correction
    :accessor correction
    :initform "")))

;*************************************************************************
;*                                                                       *
;*                 INPUT-LINE CLASS                                      *
;*                                                                       *
;*  Instance of this class hold a line of input and the number of the    *
;* input line.                                                           *
;*                                                                       *
;*************************************************************************

(defclass input-line ()
  ((input
    :initarg :input
    :accessor input
    :initform ""
    )
   (line-num
    :initarg :line-num
    :accessor line-num
    :initform 0)))

;**************************************************************************
;*                                                                        *
;*                  ERRORS CLASS                                          *
;*                                                                        *
;*  Instances of this class are designed to keep track of all the errors  *
;* that the user has made.  It has contains of of the user's input.       * 
;*                                                                        *
;**************************************************************************

(defclass errors ()
  ( (user-input
     :initarg user-input
     :accessor user-input
     :initform '())
    ( spelling
      :initarg :spelling
      :accessor spelling
      :initform '())
    (context
     :initarg :context
     :accessor context
     :initform '())
    (noun-adjective
     :initarg :noun-adjective
     :accessor noun-adjective
     :initform '())
    (want-that
     :initarg :want-that
     :accessor want-that
     :initform '())
    (direct-translation
     :initarg :direct-translation
     :accessor direct-translation
     :initform '())
    (idioms
     :initarg :idioms
      :accessor idioms
      :initform '())
    (adj-and-adj
     :initarg :adj-and-adj
      :accessor adj-and-adj
      :initform '())
    (infinitive-used
     :initarg :infinitive-used
     :accessor infinitive-used
     :initform '())
    (french-word-choice
     :initarg :french-word-choice
     :accessor french-word-choice
     :initform '())))


;**************************************************************************

(defparameter wrong-words '())

(setf wrong-words
      (make-instance 'non-dictionary 
		     :contents 
		     (list
		    ;wh-words
		      (make-instance 'wrong-word
				     :correct-form "what"
				     :other-forms '("whats" "waht"))
          
		    ;drinks
		      (make-instance 'wrong-word
				     :correct-form "drink"
				     :other-forms '("drnk" "drnik"))
		      (make-instance 'wrong-word
				     :correct-form "cold"
				     :other-forms '("clod" "cld"))
		      (make-instance 'wrong-word
				     :correct-form "beverage"
				     :other-forms '("bevrage"))
		      (make-instance 'wrong-word
				     :correct-form "coffee"
				     :other-forms '("cofee" "coffe" "cofe"))
		      (make-instance 'wrong-word
				     :correct-form "tea"
				     :other-forms '("tee"))
		      (make-instance 'wrong-word
				     :correct-form "orange"
				     :other-forms '("ornge" "oange" "oragne"))
		      (make-instance 'wrong-word
				     :correct-form "rootbeer"
				     :other-forms '("rotbeer" "rutebeer" 
						    "routbeer"))
		    ;;food  
		      (make-instance 'wrong-word
				     :correct-form "cake"
				     :other-forms '("cacke" "caik"))
		      (make-instance 'wrong-word
				     :correct-form "cheesecake"
				     :other-forms '("cheasecake" "chesecake"))
		      (make-instance 'wrong-word
				     :correct-form "ice-cream"
				     :other-forms '("ice-creem" "ice-creme"))
		      (make-instance 'wrong-word
				     :correct-form "chocolate"
				     :other-forms '("chocolat"))
		      (make-instance 'wrong-word
				     :correct-form "strawberry"
				     :other-forms '("strwberry" "strawbery" 
						    "stawberry" "strabrry" 
						    "strabery"))
		      (make-instance 'wrong-word
				     :correct-form "food"
				     :other-forms '("fod"))
		      (make-instance 'wrong-word
				     :correct-form "spaghetti"
				     :other-forms '("spagetti" "spageti" 
						    "spagheti"))
		      (make-instance 'wrong-word
				     :correct-form "ravioli"
				     :other-forms '("raviolli" "ravolli" 
						    "ravoli"))
		      (make-instance 'wrong-word
				     :correct-form "burger"
				     :other-forms '("brger" "burgr" "bugre" 
						    "berger"))
		      (make-instance 'wrong-word
				     :correct-form "cheeseburger"
				     :other-forms '("cheaseburger" 
						    "cheseburger" 
						    "cheeseberger" 
						    "cheesebrger" 
						    "cheasebrger"))
		      (make-instance 'wrong-word
				     :correct-form "cheese"
				     :other-forms '("chease" "cheeze" 
						    "cheaze"))
		      (make-instance 'wrong-word
				     :correct-form "hamburger"
				     :other-forms '("hamburgr" "hambrger" 
						    "hamberger" "hamburgre"))
		      (make-instance 'wrong-word
				     :correct-form "fries"
				     :other-forms '("freis" "frys"))
		      (make-instance 'wrong-word
				     :correct-form "pizza"
				     :other-forms '("piza"))
		      (make-instance 'wrong-word
				     :correct-form "dessert"
				     :other-forms '("desert"))
		      (make-instance 'wrong-word
				     :correct-form "medium"
				     :other-forms '("meduim" "meadium" 
						    "meaduim"))
		      (make-instance 'wrong-word
				     :correct-form "diet"
				     :other-forms '("deit"))
		      (make-instance 'wrong-word
				     :correct-form "cream"
				     :other-forms '("creem" "creme" "craem"))
		      (make-instance 'wrong-word
				     :correct-form "sugar"
				     :other-forms '("sugour" "sugra"))
		      (make-instance 'wrong-word
				     :correct-form "without"
				     :other-forms '("wihtout" "withuot" 
						    "wihtuot"))
		      (make-instance 'wrong-word
				     :correct-form "black"
				     :other-forms '("blak" "blac" "blakc" 
						    "blck"))
		      (make-instance 'wrong-word
				     :correct-form "with"
				     :other-forms '("wiht"))
		      
		      (make-instance 'wrong-word
				     :correct-form "smoking"
				     :other-forms '("smking" "smokng"))
		      (make-instance 'wrong-word
				     :correct-form "washroom"
				     :other-forms '("washrom" "washroum" 
						    "washrume"))
		      (make-instance 'wrong-word
				     :correct-form "special"
				     :other-forms '("specail"))
		      (make-instance 'wrong-word
				     :correct-form "specials"
				     :other-forms '("specails"))
		      (make-instance 'wrong-word
				     :correct-form "neither"
				     :other-forms '("niether"))
		      (make-instance 'wrong-word
				     :correct-form "nothing"
				     :other-forms '("nthing" "nohting" "hting")
				     )
		      (make-instance 'wrong-word
				     :correct-form "serve"
				     :other-forms '("srve" "sreve"))
		      (make-instance 'wrong-word
				     :correct-form "order"
				     :other-forms '("ordre"))
		      (make-instance 'wrong-word
				     :correct-form "offer"
				     :other-forms '("offre" "ofer" "ofre"))
		      (make-instance 'wrong-word
				     :correct-form "have"
				     :other-forms '("haev" "hvae" "hvea" "ahev"
						    "aveh" "avhe"))
		      (make-instance 'wrong-word
				     :correct-form "would"
				     :other-forms '("wuold" "woud" "owudl" 
						    "woudl"))
		      (make-instance 'wrong-word
				     :correct-form "could"
				     :other-forms '("cuold" "coud" "coudl"))
		      (make-instance 'wrong-word
				     :correct-form "regular"
				     :other-forms '("regluar"))
		      (make-instance 'wrong-word
				     :correct-form "want"
				     :other-forms '("wnat" "wants" "wnats"))
                
		      )))






(defparameter list-of-errors (make-instance 'errors))

 
;*********************************
;*                               *
;*  methods for errors class     *
;*                               *
;*********************************  

(defmethod determine-errors ((self errors))
  (loop for item in (user-input self) do
	(noun-adj (input item) (line-num item))
	(find-want-that (input item) (line-num item))
	(look-for-french-words (input item) (line-num item))
	(look-for-bad-translations (input item) (line-num item))
	(idiom-could-be-used (input item) (line-num item))
	(look-for-adj-and-adj (input item) (line-num item))
	(look-for-infinitives (input item) (line-num item))))
                           

(defmethod print-errors-by-category ((self errors))
  (format t "~%")
  (format t"~%Post Dialogue Analysis")
  (format t "~%")
  (format t "~%User Input:")
  (format t "~%")
  (determine-errors self)
     
  (setf (user-input self) (reverse (user-input self)))
  
  (loop for item in (user-input self) do
	(format t "~%Line ~a:  ~a" (line-num item) (input item)))
  (cond ((noun-adjective self)
	 (format t "~%")
	 (format t "~%Adjective-before-noun-errors:")
    
         (loop for item in (noun-adjective self) do
	       (format t "~%Line ~a:  Adjective:  ~a after Noun:  ~a. " 
		       (line-num item) (string-upcase (adj item)) 
		       (string-upcase (noun item)))))) 

  (cond ((spelling self)
	 (format t "~%")
	 (format t "~%Spelling mistakes:")
	 (loop for item in (spelling self) do
	       (format t "~%")
               (format t 
		       "~%There is a spelling mistake on line ~a of the input."
		       (line-num item))
	       (format t "~%The correct spelling of ~a is ~a." 
		       (string-upcase (mistake item)) (string-upcase 
						       (correction item))))))
  (format t "~%")
  (cond ((context self)
	 (format t "~%Context errors:")
	 (loop for item in (context self) do
	       (determine-jump item)
	       (determine-problem item)
              
	       (format t "~%")
             
	       (format t"~%Line: ~a ~a" (line-num item)(problem item))
	       )))
  (format t "~%")

  (format t "~%Other errors:")
  (format t "~%")
      
  (loop for item in (want-that self) do
	(format t "~%Line ~a:  ~a" (line-num item) (message item))
	(format t "~%"))
  

  (loop for item in (french-word-choice self) do
	(format t 
      "~%Line ~a:  The french word ~a was used instead of the english form ~a."
      (line-num item) (string-upcase (french-used item)) 
      (string-upcase (english-form item)))
	(format t "~%"))

  (loop for item in (idioms self) do
	(format t "~%Line ~a:  Instead of ~a," (line-num item) 
		(string-upcase (phrase-given item)))
	(format t "~%the user could have said ~a." 
		(string-upcase (actual-idiom item)))
	(format t "~%"))

  (loop for item in (noun-adjective self) do
	(format t 
	 "~%Line ~a:  The conjunction AND is between the adjectives ~a and ~a "
	 (line-num item) (string-upcase (1st-adj item)) 
	 (string-upcase (2nd-adj item)))
	(format t "~%")
	)
  
  (loop for item in (direct-translation self) do
	(format t "~%Line ~a:  " (line-num item))
	(format t "~%The phrase:  *~a* " (french-phrase item))
	(format t "should not be translated to ~% *~a*." 
		(translation-given item))
	(format t "~%Perhaps a more appropriate translation would be ~% *~a*."
		(correct-translation item))
	(format t "~%"))

  (loop for item in (infinitive-used self) do
	
	(format t "~%Line ~a: The infinitive of the verb ~a was used." 
		(line-num item) (string-upcase (verb item)))
	(format t "~%"))
      

  )

(defmethod print-by-line-number ((self errors))

  (let ((count 1)
	(index 0))
    (format t "~%POST DIALOGUE ANALYSIS")
    (format t "~%")     
    (determine-errors self)
 
    (setf (user-input self) (reverse (user-input self)))
    
    (format t "~%")
    
    (loop until (eq count (+ (length (user-input self)) 1)) do
	  (format t "~%Line ~a:  ~a" count (input (nth index 
						       (user-input self))))
	  (format t "~%")
	  (loop for item in (spelling self) do
		(cond ((eq count (line-num item))
		       (format t "~%The correct spelling of ~a is ~a." 
			       (string-upcase (mistake item)) (string-upcase 
							       (correction item
									   )))
		       (format t "~%"))))
	  (loop for item in (context self) do
		(cond ((eq count (line-num item))
                
		       (determine-jump item)
		       (determine-problem item)
              (format t"~%~a" (problem item))
	      (format t "~%"))))

	  (loop for item in (want-that self) do
		(cond ((eq count (line-num item))
		       (format t "~%~a" (message item))
		       (format t "~%"))))

    
	  (loop for item in (french-word-choice self) do
		(cond ((eq count (line-num item))
		       (format t 
		"~%The french word ~a was used instead of the english form ~a."
		(string-upcase (french-used item)) (string-upcase 
						    (english-form item)))
		       (format t "~%"))))

     
	  (loop for item in (direct-translation self) do
		(cond ((eq count (line-num item))
		       (format t "~%The phrase:  *~a* " (french-phrase item))
		       (format t "should not be translated to ~% *~a*." 
			       (translation-given item))
		       (format t 
		   "~%Perhaps a more appropriate translation would be ~% *~a*."
		   (correct-translation item))
		       (format t "~%"))))

	  (loop for item in (noun-adjective self) do
		(cond ((eq count (line-num item))
		       (format t "~%Adjective:  ~a after Noun:  ~a."
			       (string-upcase (adj item)) 
			       (string-upcase (noun item)))
		       (format t "~%")
		       )))
	  
	  (loop for item in (idioms self) do
		(cond ((eq count (line-num item))
		       (format t "~%Instead of ~a,"
			       (string-upcase (phrase-given item)))
		       (format t "~%you could have said ~a." 
			       (string-upcase (actual-idiom item)))
		       (format t "~%")
		       )))
	  
	  (loop for item in (adj-and-adj self) do
		(cond ((eq count (line-num item))
		       (format t 
		   "~%The conjunction AND is between the adjectives ~a and ~a."
		   (string-upcase (1st-adj item)) (string-upcase (2nd-adj item)
								 ))
		       (format t "~%")
		       )))


	  (loop for item in (infinitive-used self) do
		(cond ((eq count (line-num item))
		     (format t "~%The infinitive form of the verb ~a was used."
			     (string-upcase (verb item))))))
    
	  (incf count 1)
	  (incf index 1)
	  (format t "~%")
	  ))) 
      
      



;*************************************
;*                                   *
;*  methods for non-dictionary class *
;*                                   *
;*************************************


(defmethod non-look-up ((self non-dictionary) index input-in-word-segments 
			input-line-num)
;;  This method is similar to the look-up method for words in the normal
;; dictionary.  This method checks for spelling mistakes.  If some are found
;; they are recorded.
  (let* ((curword '())
	 (found '())
	 (error-found '())
	 (substring (nth-cdr index input-in-word-segments)))
    (loop for current-word in (contents self) until found do
	  (setf found (word-matches current-word substring))
	  (cond (found ;;(format t "~%Found a mistake with ~a" found)
		 (setf error-found (make-instance 'spelling-mistake
						  :mistake (form-given  found)
						  :correction (correct-form  
							       found)
						  :line-num input-line-num))
                    
		 (push error-found (spelling list-of-errors)) 
               
		 )))
    ))



(defmethod initialize ((self non-dictionary))
  (loop for w in (contents self) do
	;;(format t "~%Looping with wrong-word: ~a" w)
        (loop for i from 0 below (length (other-forms w)) do
	 ;;(format t "~% looping with other-form: ~a" (nth i (other-forms w)))
	      (setf (nth i (other-forms w))
		    (walk-string recognize-words (nth i (other-forms w))))))
  self)



;****************************************
;*                                      *
;*  methods for wrong-word class        *
;*                                      *
;****************************************

(defmethod word-matches ((self wrong-word) substring-input)
  (let ((results '())
	(found '()))

;;(format t "~%Inside wrong-word-matches with ~a and ~a" self substring-input)

    (setf found (lines-up (walk-string recognize-words
				       (string-downcase (string 
							 (correct-form self))))
			  substring-input))
    
    (cond (found (setf results
		       (list found (word-copy self :actual-input
					      (string-downcase
					       (string (correct-form self))) 
					      ) )))
	  (t (loop for w in (other-forms self) until found do
		   (setf found (lines-up w substring-input))
		   (if found (setf results
				   
                                   (word-copy self
					      :actual-input (list-to-string w))
				   )))))
    results))

(defmethod word-copy ((self wrong-word) &key actual-input)
  ( let ((a
	  (make-instance 'wrong-word
			 :form-given actual-input
			 :correct-form (correct-form self)
			 :other-forms (other-forms self))))
    a))

(defmethod print-object ((self wrong-word) stream)
  (format stream "<~a: ~a>" (class-name (class-of self)) (correct-form self))) 


;****************************************
;*                                      *
;*  methods for spelling-mistake class  *
;*                                      *
;****************************************

(defmethod mistake-copy ((self spelling-mistake) &key)
  (let ((a
	 (make-instance 'spelling-mistake
			:mistake (mistake self)
			:correction (correction self)
			:line-num (line-num self))))
    a))










