;######################################################################
;#                                                                    #
;#                     STEP.LSP                                       #
;#                                                                    #
;#    CONTENTS:                                                       #
;#       Classes:  mystep, results                                    #
;#       Functions:  between (a b c)                                  #
;#       Comment:  I cannot my the life of me figure out what the     #
;#                 the bindings stuff is for.  That's is something I  #
;#                 will never understand.                             #
;#                                                                    #
;######################################################################


;**************************************************************
;*                                                            *
;*                  MYSTEP CLASS                              *
;*   This is actually the step class.  I LISP made me rename  *
;*   it.                                                      *
;*  Input-Conditions:  set of conditions concerning the input *
;*                     that must be true in order for the     *
;*                     step to be applicable                  *
;*  Situation-Conditions:  set of conditions concerning the   *
;*                         current situation that must be true*
;*                          in order for the step to be       *
;*                        applicable.                         *
;*  Results:  what is obtained when tests are preformed       *
;*  Name:  the step's name                                    *
;*  My-action:  the code that is executed is the step is      *
;*              chosen                                        *
;*  Substeps and successfully substeps:  not used at this time*
;**************************************************************

(defclass mystep ()
    (;(type
     ;   :initarg :type
     ;   :accessor type
     ;   :initform 'scenario)
     (input-conditions
      :initarg :input-condition
      :accessor input-condition
      :initform nil)
     (situation-conditions
      :initarg :my-condition
      :accessor my-condition
      :initform  (spscorec :sc 0))
     (results
        :initarg :results
        :accessor results
        :initform (make-instance 'results :success t :score 0))
     (input-results        
      :initarg :input-results
      :accessor input-results
      :initform (make-instance 'results :success t :score '()))
     (name 
        :initarg :name
        :accessor name
        :initform "a step")
     (my-action
        :initarg :my-action
        :accessor my-action
        :initform #'(lambda (self waiter) t))
     (substeps
      :initarg :substeps
      :accessor substeps
      :initform '())
     (successful-substeps
      :initarg :sucessful-substeps
      :accessor successful-substeps
      :initform '())
     ))


(defmethod is-applicable ((self mystep) restaurant)
;;  Decides wether a step is applicable.  If the waiter has input and the
;; step has input-condition than the test is performed on those conditions.
;; If there are no input-condition, nothing is done.  Otherwise, if
;; there are input-condition and no input, the input-results are step to
;; false.  Similar, if they are situation-conditions, the tests are performed
;; on them.  Then the method total-success is invoked in order to find out
;; if both the input-results and the results are a success.

   (cond ((and (input (waiter restaurant))
               (input-condition self))
          (setf (input-results self)
                (do-test (input-condition self)
                   restaurant self)))
         ((null (input-condition self)))
         (t (setf (success (input-results self)) nil)))
   (when debugp (format t "~%inside mystep::is-applicable"))
   (when (my-condition self)
      (setf (results self)
            (do-test  (my-condition self) 
               restaurant self)))
   (if (total-success self)
      (loop for sub in (substeps self) do
         (when (is-applicable sub restaurant)
            (append (successful-substeps self) (list sub)) ))
      (setf (successful-substeps self) nil))
   (when debugp (format t "~%  results after testing mystep::is-applicable): ~a ~%~a" 
                   (results self)
                   (input-results self)))
   (total-success self))



(defmethod total-success ((self mystep))
;; Returns true if both the input-results and the results are a success
   (and (success (input-results self))
        (success (results self))))


(defmethod binding-value ((self mystep) slot-name)
   (let ((found nil))
      (loop for pair in (bindings (results self)) do
         (if (eq (car pair) slot-name)
            (setf found (cadr pair))))
      (loop for pair in (bindings (input-results self)) do
         (if (eq (car pair) slot-name)
         (setf found (cadr pair))))
      found))

(defmethod binding-values ((self mystep) slot-name)
   (let ((found nil))
      (loop for pair in (bindings (results self)) do
         (if (equal (car pair) slot-name)
            (push (second pair) found)))
      (loop for pair in (bindings (input-results self)) do
         (if (equal (car pair) slot-name)
         (push (second pair) found )))
      found))

(defmethod do-step ((self mystep) restaurant)
;;  Executes the code in the my-action of the step and sets the 
;; expecting-input of the waiter to nil.
   (when debugp (format t "~%   Doing step ~a" self))
  (setf (expecting-input (waiter restaurant)) nil)
  (apply (my-action self) (list self restaurant))

  )

(defmethod make-copy ((self mystep)) :after
  :documentation "This method is intended for use in the history list only.
     Everything will be shared, except for the results. The new step will have
     a different set of results than the old step."
  (make-instance 'mystep
     :name (name self)
     :results (make-copy (results self))
     :input-results (make-copy (input-results self))
     :my-condition (my-condition self)
     :my-action (my-action self)))

(defmethod total-score ((self mystep))
   (+ (length (score (input-results self))) (score (results self))))

(defmethod print-object ((self mystep) stream)
  (format stream "<mystep: ~a>" (name self)))

(defmethod start-index ((self mystep))
   "Returns the index of the first word matched by a step template."
   (first (score (input-results self))))

(defmethod end-index ((self mystep))
   "Returns the index of the last word matched by a mystep template."
   (first (last (score (input-results self)))))

(defmethod strictly-precedes ((step1 mystep) (step2 mystep))
    (< (end-index step1) (start-index step2)))

(defmethod step-weight ((self mystep))
      (length (score (input-results self))))


;------------------------------------------------------------------------------
;        Comparing steps
;------------------------------------------------------------------------------
(defmethod subsumes ((self mystep) opponent)
    (subsetp (score (input-results opponent)) (score (input-results self))))

(defmethod intertwined ((self mystep) opponent)
   ;;; a step is intertwined with its opponent if the pattern of words that 
   ;;; matched overlaps more than one of the words the opponent matched (but 
   ;;; one does not subsume the other: this is tested for by subsumes).
   (let* ((my-score (score (input-results self)))
          (other-score (score (input-results opponent)))
          (my-start (first my-score))
          (my-end (first (last my-score)))
          (other-start (first other-score))
          (other-end (first (last other-score))))
      (or (and (between my-start other-start other-end)
               (> my-end other-end))
          (and (between my-end other-start other-end)
               (< my-start other-start)))      
    )
)

(defun between (a b c)
   ;;; a is between b and c 
   (and (> a b)
        (< a c)))
   
;------------------------------------------------------------------------------
;        The RESULTS CLASS
;  
;------------------------------------------------------------------------------

(defclass results ()
  ((bindings
   :initarg :bindings
   :initform '()
   :accessor bindings)
  (score                       
   :initarg :score                 
   :initform 1                 
   :accessor score)            
       ;;; let's say that the "score" for word-results is not a numerical score as it 
       ;;; is in the case of non-word results, but instead is the list of words that 
       ;;; bound (eg. '(1 2 3 5))
  (success
   :initarg :success
   :initform '()
   :accessor success)
  (remaining-input
   :initarg :remaining-input
   :accessor remaining-input
   :initform '()))

  (:documentation " The RESULTS object holds the results of testing if a step
      is applicable. (i.e. whether the applicability conditions hold, what the 
      score for the step is (for choosing the best next step to apply), and 
      any bindings that were created, if there was input and it was used."))

(defmethod add-bindings ((self results) new-bindings)
 ; (when debugp (format t "~%     Adding binding: ~a to: ~a" 
 ;                 new-bindings (bindings self)))
  (if (and new-bindings (not (null (car new-bindings))))
   (setf (bindings self) (append (bindings self) new-bindings))))

(defmethod add-score ((self results) new-score)
   ;;;USE THIS METHOD TO INCREASE THE NUMERICAL SCORE OF A NON-INPUT STEP RESULT
  (setf (score self) (+ (score self) new-score)))

(defmethod add-to-score ((self results) new-score)
   ;;;USE THIS METHOD TO ADD MATCHED WORDS TO THE SCORE
   (when (integerp (score self))
      (setf (score self) nil))
   (setf (score self) (flatten (append (list new-score) (score self) )))
                         )

(defmethod remove-bindings-from-remaining-input ((self results) bindings)
   (loop for wd in bindings do
        (setf (remaining-input self)
            (remove (second wd) (remaining-input self)))))

(defmethod remove-words ((self results) word-numbers offset)
   (let ((new-input '())
         (actual-numbers '()))
      (setf actual-numbers (mapcar #'(lambda (x) (- x offset)) word-numbers))
     (loop for index from 0 below (length (remaining-input self)) do
         (unless(member index actual-numbers) 
            (setf new-input (cons (nth index (remaining-input self))
                               new-input))))
      (format t "~%new-input: ~a" new-input)
      (setf (remaining-input self) (reverse new-input))))

(defmethod clear-results ((self results))
  (setf (bindings self) '())
  (setf (score self) 0)
  (setf (success self) nil)
  (setf (remaining-input self) '()))

(defmethod get-binding ((self results) variable-name)
  (let ((done '())
        (i 0)
        (result '()))
    (loop until done do
          (cond ((>= i (length (bindings self))) (setf done t))
                ((eq variable-name (car (nth i (bindings self))))
                 (setf result (cadr (nth i (bindings self))))
                 (setf done t))
                (t (incf i 1))))
    result))
 
(defmethod use-word ((self results) word)
   (setf (remaining-input self) (remove word (remaining-input self))))

(defmethod make-copy ((self results)) :after
  (let ((the-copy (make-instance 'results
                      :bindings (copy-list (bindings self))
                      :score (score self)
                      :remaining-input (copy-list (remaining-input self)) )))
    (if (success self)
        (setf (success the-copy) t)
      (setf (success the-copy) nil))
    the-copy))

             

(defmethod print-object ((self results) stream)
  (format stream "<step-results: (~a ~a ~a ~a)>" 
          (success self) 
          (score self) 
          (bindings self)
          (remaining-input self)))









