;;###################################################################
;;#                                                                 #
;;#                SUBSCENARIO2.SCE                                 #
;;#                                                                 #
;;#   CONTENTS:                                                     #
;;#      parameters:                                                # 
;;#          subscenarios:  coffee-scenario                         #
;;#                             steps:  ask-if-cream-or-sugar,      #
;;#                                     just-sugar, do-not-get-sugar#
;;#                                   just-cream, do-not-get-cream  #
;;#                                    get-neither, get-both,       #
;;#                                     black-coffee,               #
;;#                                   cream-and-sugar,              #
;;#                                   no-cream-or-sugar             #
;;#                         pop-scenario                            #
;;#                            steps:  ask-if-diet-or-regular       #
;;#                                    get-diet, get-regular        #
;;#                         steak-scenario                          #
;;#                           steps:                                #
;;#                                  ask-how-done, list-choices     #
;;#                                  get-rare, get-medium,          #
;;#                                  get-well-done, medium-I-guess  #
;;#                                                                 #
;;###################################################################

;;  Subscenarios can be attached to orderable-item.  When an item is
;; ordered that has a subscenario is ordered, that subscenario is pushed
;; onto the scenario stack.  This is to be used in place of the slot-scenarios.
;; Subscenario allow the types of questions and acceptable input to be better
;; suited to the individual items. 


(defclass subscenario (scenario)
  ((item :initarg :item
	 :initform nil
	 :accessor item)))


;;*********************************
;;**                             **
;;**    Coffee                   **
;;**                             **
;;*********************************

(defparameter ask-if-cream-or-sugar 
  (make-instance 'mystep
		 :name "ask-if-cream-or-sugar"
		 :my-condition (spscorec :sc 1 :nec 
					 (notc 
					  (historyc "ask-if-cream-or-sugar")))
		 :my-action 
		 #'(lambda (self restaurant)
		     (say (waiter restaurant) 
			  "~%Would you like cream and sugar for your coffee?")
		     (setf (expecting-input (waiter restaurant)) t))))


(defparameter get-both 
  (make-instance 'mystep
		 :name "get-both"
		 :input-condition (orc (list (wordc :nec '(yes))
					     (wordc :nec '(both))))
		 :my-condition (lambdac #'(lambda (restaurant mystep)
					    (prev-step-matches 
					     restaurant
					     '("ask-if-cream-or-sugar") 2)))
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~%Cream and sugar?  No problem!"))))

(defparameter get-neither 
  (make-instance 'mystep
		 :name "get-neither"
		 :input-condition (orc (list (wordc :nec '(no))
					     (wordc :nec '(none))
					     (wordc :nec '(nothing))))
		 :my-condition (lambdac #'(lambda (restaurant mystep)
					    (prev-step-matches 
					     restaurant 
					     '("ask-if-cream-or-sugar") 2)))
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~%No cream or sugar?  Okay"))))

(defparameter cream-and-sugar 
  (make-instance 'mystep
		 :name "cream-and-sugar"
		 :input-condition (orc (list (wordc :nec '(cream and sugar))
					     (wordc :nec '(sugar and cream))))
		 :my-condition (spscorec :sc 3)
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~%Cream and sugar?  No problem!"))))

                     
(defparameter no-cream-or-sugar 
  (make-instance 'mystep
		 :name "no-cream-or-sugar" 
		 :input-condition (orc (list 
					(wordc 
					 :nec '(without-cream and 
							      without-sugar))
					(wordc :nec '(without-cream or sugar))
					(wordc :nec '(nothing))
					(wordc :nec '(without-sugar or cream)))
				       )
		 :my-condition (spscorec :sc 3)
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~%No cream or sugar?  No problem."))))


                     
                              

(defparameter just-cream 
  (make-instance 'mystep
		 :name "just-cream"
		 :input-condition (wordc :nec '(cream)
					 :sup '(without-sugar))
		 :my-condition (spscorec :sc 2)
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) "~%Cream?  Sure!"))))



(defparameter do-not-get-cream 
  (make-instance 'mystep
		 :name "do-not-get-cream"
		 :input-condition (wordc :nec '(without-cream))
		 
		 :my-condition (spscorec :sc 2 )
		 
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~%No cream?  No problem!")
				)))



                  

(defparameter just-sugar 
  (make-instance 'mystep
		 :name "just-sugar"
		 :input-condition (wordc :nec '(sugar)
					 :sup '(without-cream))
		 
		 :my-condition (spscorec :sc 2)
		 
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~%Just sugar?  Sure!"))))



(defparameter do-not-get-sugar 
  (make-instance 'mystep
		 :name "do-not-get-sugar"
		 :input-condition (wordc :nec '(without-sugar))
		 
		 :my-condition (spscorec :sc 2)
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~%No sugar?  No problem!")
				)))


  
(defparameter black-coffee 
  (make-instance 'mystep
		 :name "black-coffee"
		 :input-condition (wordc :nec '(black))
		 :my-condition (spscorec :sc 4)
		 
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) "~%Black?  Sure!"))))

 

(defparameter coffee-scenario 
  (make-instance 'subscenario
		 :name "subscenario"
		 :parent order-scenario
		 
		 :steps (list washroom ask-if-cream-or-sugar just-sugar 
			      do-not-get-sugar just-cream do-not-get-cream 
			      get-neither get-both black-coffee 
			      cream-and-sugar no-cream-or-sugar)
		 :end-condition (orc (list (historyc "get-neither")
					   (historyc "black-coffee")
					   (historyc "get-both")
					   (historyc "just-cream")
					   (historyc "just-sugar")
					   (historyc "do-not-get-sugar")
					   (historyc "do-not-get-cream")
					   (historyc "cream-and-sugar")
					   (historyc "no-cream-or-sugar")))))


;;*******************************
;;**                           **
;;**       Soft-drink          **
;;**                           **
;;*******************************




(defparameter ask-if-diet-or-regular 
  (make-instance 'mystep
		 :name "ask-if-diet-or-regular"
		 :my-condition (spscorec 
				:sc 2 
				:nec 
				(andc :nec 
				      (list
				       (notc (historyc "get-diet"))
				       (notc (historyc "get-regular"))
				       (notc (historyc "ask-if-diet-or-regular"
						       )))))
		 :my-action 
		 #'(lambda (self restaurant)
		     (say (waiter restaurant) 
			  "~%For your ~a, would you like diet or regular?" 
			  (reference-name (item (current-scenario 
						 (waiter restaurant)))))
		     (setf (expecting-input (waiter restaurant)) t)
            
		     )))

(defparameter get-diet 
  (make-instance 'mystep
		 :name "get-diet"
		 :input-condition (wordc :nec '(diet))
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) "~%Diet? Sure!"))))

(defparameter get-regular 
  (make-instance 'mystep
		 :name "get-regular"
		 :input-condition (wordc :nec '(regular))
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) "~%Regular? Sure!"))))

(defparameter pop-scenario 
  (make-instance 'subscenario
		 :name "subscenario"
		 :steps (list washroom ask-if-diet-or-regular get-diet 
			      get-regular)
		 :end-condition (orc (list (historyc "get-regular")
					   (historyc "get-diet")))
		 ))


;;************************************
;;**                                **
;;**          Steak                 **
;;**                                **
;;************************************

           


(defparameter ask-how-done 
  (make-instance 'mystep
		 :name "ask-how-done"
		 :my-condition (spscorec 
				:sc 2 
				:nec 
				(andc :nec (list
					    (notc (historyc "ask-how-done"))
					    (notc (historyc "list-choices"))
					    (notc (historyc "get-rare"))
					    (notc (historyc "get-medium"))
					    (notc (historyc "get-well-done"))))
				)
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~%How would you like your steak done?")
				(setf (expecting-input (waiter restaurant)) t)
				)))

(defparameter list-choices 
  (make-instance 'mystep
		 :name "list-choices"
		 :my-condition (andc :nec (list (historyc "ask-how-done")
						(notc (historyc "get-rare"))
						(notc (historyc "get-medium"))
						(notc (historyc "get-well-done"
								))
						(notc (historyc "list-choices")
						      )))
		 :my-action 
		 #'(lambda (self restaurant)
		     (say (waiter restaurant) 
	      "~%I'm sorry, we only cook our steaks rare, medium or well-done. 
 Which would you prefer?")
		     (setf (expecting-input (waiter restaurant)) t))
		 ))

(defparameter get-rare 
  (make-instance 'mystep
		 :name "get-rare"
		 :input-condition (wordc :nec '(rare))
		 :my-condition (spscorec :sc 4)
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~% One rare steak coming right up!"))
		 ))
         

(defparameter get-medium 
  (make-instance 'mystep
		 :name "get-rare"
		 :input-condition (wordc :nec '(medium))
		 :my-condition (spscorec :sc 4)
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~% One medium steak coming right up!"))
		 ))

(defparameter get-well-done 
  (make-instance 'mystep
		 :name "get-rare"
		 :input-condition (wordc :nec '(well-done))
		 :my-condition (spscorec :sc 4)
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
				     "~% One well-done steak coming right up!")
				)
		 ))

(defparameter medium-I-guess 
  (make-instance 'mystep
		 :name "medium-I-guess"
		 :input-condition (andc :nec 
					(list (notc (wordc :nec '(rare)))
					      (notc (wordc :nec '(medium)))
					      (notc (wordc :nec '(well-done))))
					)
		 :my-condition (andc :nec (list
					   (notc (historyc "get-rare"))
					   (notc (historyc "get-medium"))
					   (notc (historyc "get-well-done"))
					   (historyc "list-choices")))
		 :my-action #'(lambda (self restaurant)
				(say (waiter restaurant) 
     "~%Hmm...I am a little bit confused. I guess I will tell the cook medium."
     ))))


(defparameter steak-scenario 
  (make-instance 'subscenario
		 :name "subscenario"
		 :steps (list washroom ask-how-done list-choices get-rare 
			      get-medium get-well-done medium-I-guess)
		 :end-condition (orc (list (historyc "get-rare")
					   (historyc "get-medium")
					   (historyc "get-well-done")
					   (historyc "medium-I-guess")))
		 ))








