(defun random-pick (list)
   (nth (random (length list)) list))

(defun increment (int)
   (+ int 1))

(defun append-char (string char)
   (format nil "~a~a" string char))

(defun string-to-list (string)
   (let ((list '()))
      (loop for i downfrom (- (length string) 1) to 0
           do (push (aref string i) list))
      list))

(defun list-to-string (list)
   (let ((string ""))
      (dotimes (i (length list))
         (setf string (append-char string (nth i list))))
      string))


(defun lines-up (a b)
   :documentation "(lines-up a b) returns the length of a (i.e. the amount of
    a that matched b)  if a is a sublist of b and a starts at the start of b."
  (let ((found t))
;;;  (when debugr (format t "~%inside lines-up with ~a and ~a" a b))
   (loop for i from 0 below (length a) do 
         (unless (equal (nth i a) (nth i b))
                 (setf found nil)))
   (if found (setf found (length a)) found)))

(defun nth-cdr (index list)
   :documentation "return the tail of the list. The 'tail' is what follows
   the list after and including the indexth item in the list. For example,
   (nth-cdr 0 '(a b c)) will return '(a b c);
   (nth-cdr 1 '(a b c)) will return '(b c), and so on. 
   (note: (nth-cdr 5 '(a b c)) will return '())"
;;;(when debugr (format t "~%inside nth-cdr with ~a and ~a" index list))
  (if (<= index 0)
      list
      (nth-cdr (- index 1) (cdr list))))

(defmethod kind-of ((subclass symbol) (superclass symbol))
    :documentation "returns true if the first argument is a subclass of the second"
      (loop with class = (find-class subclass)
	    until (or (eq (class-name class) 'standard-object)
		      (eq (class-name class) t))
	    do 
	    (when debug2p (format t "~%(In Kind-of) Class:~a ~a" (class-name class) superclass))
	    (when  (eq  superclass (class-name class))
	      (return t)) 
	    (setf class (first (clos:class-direct-superclasses class)))))

(defun wait (seconds) 
  (let ((start-time (+ (get-universal-time) seconds)))
    (loop until (> (get-universal-time) start-time) do
        ;  (format t "~% ~a" (listen *terminal-io*))
       (+ 1 1)
       )))

(defun get-most-frequent (list)
  :documentation "return the most frequent element of the list. If there is a
    tie, the first of the tied elements that occurs in the list is returned.
    Nil is returned if the list is empty.
    Used by the Waiter::get-best-scenario method. "
  (if list
      (let ((unique-elements '())
            (scores '())
            (temp '())
            (index 0))
        (loop for i in list do
              (setf temp (member i unique-elements))
              (cond (temp (setf index (- (length unique-elements) (length temp)))
                          (setf (nth index scores) (+ (nth index scores) 1)))
                    (t (push i unique-elements) 
                       (push 1 scores))))
        (setf scores (reverse scores))
        (setf unique-elements (reverse unique-elements))
        (setf index (-  (length scores) (length (member (apply 'max scores) scores))))
        (nth index unique-elements))))
  
(defparameter yes t)
(defparameter no nil)
  
(defun top (stack)
         (car stack))
