(defclass waiter (person)
    ((name
        :initarg :name
        :accessor name
        :initform "Wally")
     (scenarios
        :type list
        :initarg :scenarios
        :accessor scenarios
        :initform nil)
     (scenario-stack
        :type list
        :initarg :scenario-stack
        :accessor scenario-stack
        :initform nil)
     (history
        :accessor history
        :initform nil)
     (input
        :accessor input
        :initform '()
        :initarg :input)
     (input-expectations
        :accessor input-expectations
        :initform '()
        :initarg :input-expectations)
        ;;;NOTE: INPUT EXPECTATIONS ARE THE SCENARIO EXPECTATIONS THAT CAN BE EXTRACTED FROM THE INPUT.
     (expecting-input
        :accessor expecting-input
        :initform nil
        :initarg :expecting-input)
        ;;; Expecting-input tells whether or not the last step expects or allows a reply. 
     (current-scenario 
        :initarg :current-scenario
        :accessor current-scenario
        :initform nil)
     (dictionary 
        :initarg :dictionary
        :accessor dictionary
        :initform nil)))


(defmethod current-scenario-expected ((self waiter) a-scenario )
     ;;; Return true if the current scenario or one of its ancestors is in the
     ;;; list of input-expectations. (a helper function for do-likeliest-scenario)
     (cond ((null a-scenario)  nil)
           (t (or (let ((found nil))
                     (loop for i in (input-expectations self) do
                        (if (equal a-scenario i) (setf found t)))
                     found)
                  (current-scenario-expected self (parent a-scenario) )))))
  
(defmethod do-likeliest-scenario ((self waiter) restaurant)
  :documentation "Get-best-scenario tries to find the best scenario. First
     preference is for the scenario on the top of the stack; next preference
     is for the scenario that occurs most frequently in the input-expectations."
   (with-slots (current-scenario scenario-stack expecting-input input history) self

   (let ((current-scenario-applies '())
         (other-scenarios '())
        (temp '()))
        
      ;;;Initialize the current scenario (if it is not already initialized)  
      (if (null current-scenario)
         (setf current-scenario (pop scenario-stack)))
      (incf *TRAP*)
    (loop until (or (> *TRAP* *MAXTRAP*) (null current-scenario)) do

               (cond (expecting-input
                 (setf expecting-input nil)
                  (do-step listen restaurant)
       ;;            (format t "~%DOING CURRENT-SCENARIO: ~A" current-scenario)
         ;;          (format t "~%   Input is: ~a" input)

                  (push current-scenario history)

                  (do-next-steps current-scenario restaurant)
                 
                  (loop for i in (input-expectations self) do
   ;;                   (format t "~%DOING ANOTHER SCENARIO: ~A" i)
     ;;                 (format t "~%   Input is: ~a" input)
                   (if (and (not (eq i current-scenario)) (not (equal "attributes-scenario" (name current-scenario))))
                     (pushnew i other-scenarios)))
;;   (format t "~%other-scenarios: ~a" other-scenarios)
  ;; (format t "~%current-scenario: ~a" current-scenario)   
 ;;  (format t "~%other-scenarios: ~a" other-scenarios)
                  (loop for s in other-scenarios do
                     (push current-scenario history)
                     (do-next-steps s restaurant))

 (setf temp (pop scenario-stack))
 (cond ((and temp (equal (name temp) "subscenario")))     
      (t            (setf (input self) nil)
))
(push temp scenario-stack))
              
               (t (unless  
                             (do-next-steps current-scenario restaurant)
                          
                     (unless (is-finished current-scenario restaurant)
          (say self "~% Hmm..  I am confused.  Oh well.."))

                     (setf current-scenario (pop scenario-stack))
                     )))
                   (if (and current-scenario 
                        (is-finished current-scenario restaurant))
                  (setf current-scenario (pop scenario-stack)))

         )
      )
      )
   )
                       
                     
  
(defmethod get-scenario-from-input ((self waiter) input-expectations  restaurant)
    ;Start with the most frequently-occurring (or the first, if they are equal)
    ;input-expectation. If it's applicable, accept it; otherwise, try again with
    ;the next most frequently-occurring input-expectation until an applicable 
    ;scenario is found. If no applicable scenarios are found, take the first one
    ;that had been found and ask if it's the right one. If it's not, ignore the
    ;input and pick the first scenario on the stack. The scenario is placed into
    ;the current scenario of the waiter.
    ; NOTE: if the scenario was on the waiter's stack, it gets removed. This prevents
    ;the waiter from trying to play the same scenario over and over again.
   (let ((done '())
         (exp input-expectations))
      (loop until done do
         (cond (exp 
                  (setf (current-scenario self) (get-most-frequent exp)) 
                  (cond ((is-applicable (current-scenario self) restaurant)
                         (when debugp 
                            (format t "~% found a scenario: ~a" (current-scenario self)))
                         (setf (scenario-stack self) 
                               (remove (current-scenario self) (scenario-stack self)))
                         (when debugp (format t "~% New state of scenario-stack: ~a"
                                         (scenario-stack self)))
                         (setf done t))
                        (t (setf exp
                                 (remove (current-scenario self) exp)))
                        ))
               (t (setf (current-scenario self) (pop (scenario-stack self)))
                  (setf done t))))))
  
; Would it be appropriate to put the other scenarios from input-expectations
; on the stack too? Otherwise, how will we handle "coke and fries"?
; (or maybe we should be using some conditional information about which scenario
; applies -> the scenario's tests? (look for the best one in the input set, 
; test if it applies, and so on until an applicable scenario is found. 
; That sounds pretty reasonable. I'll have to go back and add conditions to
; scenarios now, I guess...
 
(defmethod last-step ((self waiter))
  (nth-last-item self 'mystep 1))

(defmethod nth-last-step ((self waiter) n)
   (nth-last-item self 'mystep n))

(defmethod last-scenario ((self waiter ))
  (nth-last-item self 'scenario 1))

(defmethod nth-last-item ((self waiter) itemtype n)
  :documentation "return nth last scenario or step (i.e. itemtype) that was just used.
Start counting from 1"
  (let ((return-value nil)
        (done nil)
        (i 0)
        (i-count 0))
    (loop until done do
      ; (format t "~%checking ~ath word" i)
       (when (typep (nth i (history self)) itemtype)
                  (incf i-count 1)
                 (when (= i-count n)
                    (setf done t)
                    (setf return-value (nth i (history self)))
                   ))
       (incf i 1)
       (if (>= i (length (history self)))
          (setf done t)))
     return-value
        ))

(defmethod parse ((self waiter) input-string)
  :documentation "walk-string separates the string into words (i.e
     a list of characters - a word has been found if a space or
     other separator has been found.). For each of those words, 
     look up the word in the dictionary of words, which is created
     within the dictionary file."

  (let* ((input-to-look-up (walk-string 
                              recognize-words 
                              (string-downcase input-string)))
        (index 0)
        (end-condition (length input-to-look-up))
        (results '()))

     (setf (input self) nil)
     (loop until (eq index end-condition) do  
        (setf results (look-up (dictionary self) index input-to-look-up))
        (format t "~%results are: ~a" results)
        (cond (results (setf (input self)  
                          (append (input self) (cdr results)))
                       (setf index (first results)))
              (t (setf (input self) 
                   (append (input self) (list (nth index input-to-look-up))))
                 (incf index 1))))))

(defmethod calculate-scenarios-from-input ((self waiter))
   :documentation "calculate-scenarios-from-input looks at the parsed input
    that is present in the waiter's input and records each relevant scenario
    mentioned in that input. These are stored in the waiter's list of 
    input-expectations"
    (let ((results '()))
          (loop for n in (input self) do
             (unless (listp n) 
             (setf results (append (relevant-scenarios n) results))))
       (setf (input-expectations self) results)))


(defmethod record ((self waiter) event)
  (when event
   (when debugp (format t "~%Recorded event: ~a" event))
   (push (make-copy event) (history self))))

(defmethod already-done ((self waiter) event)
   (member event (history self)))

(defmethod unused-input-remains ((self waiter))
  :documentation "input only counts as unused if it points to a scenario. If
   it doesn't point to a scenario, it's not considered important enough to try
   to find a scenario for."
  (let ((found '())
        (input  (remaining-input (results (last-step self)))))
    (defun unparsed (x) (listp x))
    (loop for w in input do
          (unless (unparsed w)
            (when (relevant-scenarios w)
	      (setf found t))))
    found))

(defmethod get-remaining-input ((self waiter))
  (setf (input self) (remaining-input (results (last-step self)))))

(defmethod say ((self waiter) &rest args)
   (apply #'format (cons t args))
   (apply #'format (cons transcript args))
   )











